@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2006 Imendio AB
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,8 @@
  *          Xavier Claessens <xclaesse@gmail.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
 
@@ -29,10 +30,10 @@
 #include <glib/gi18n.h>
 #include <gobject/gvaluecollector.h>
 
-#include "libgossip-marshal.h"
+#include "gossip-account.h"
 #include "gossip-utils.h"
 
-#include "gossip-account.h"
+#include "libgossip-marshal.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_ACCOUNT, GossipAccountPriv))
 
@@ -42,7 +43,6 @@ struct _GossipAccountPriv {
 	gint               unique_id;
 
 	GossipAccountType  type;
-	gchar             *id;
 	gchar             *name;
 	gboolean           auto_connect;
 	gboolean           use_proxy;
@@ -51,10 +51,9 @@ struct _GossipAccountPriv {
 };
 
 typedef struct {
-	GossipAccountParamFunc  callback;
-	gpointer                user_data;
-	GossipAccount          *account;
-} GossipAccountParamData;
+	GossipAccountType  account_type;
+	GList             *params;
+} GetParamsData;
 
 static void           account_class_init            (GossipAccountClass     *class);
 static void           account_init                  (GossipAccount          *account);
@@ -78,26 +77,21 @@ static void           account_param_get_valist      (GossipAccount          *acc
 						     va_list                 var_args);
 static void           account_param_get_all_foreach (gchar                  *param_name,
 						     GossipAccountParam     *param,
-						     GossipAccountParamData *data);
+						     GetParamsData          *data);
 static void           account_param_free            (GossipAccountParam     *param);
+const gchar *         account_param_name_convert    (GossipAccountType       type,
+						     const gchar            *param_name);
 static void           account_set_type              (GossipAccount          *account,
 						     GossipAccountType       type);
 
 enum {
 	PROP_0,
 	PROP_TYPE,
-	PROP_ID,
 	PROP_NAME,
 	PROP_AUTO_CONNECT,
-	PROP_USE_PROXY
+	PROP_USE_PROXY,
 };
 
-enum {
-	CHANGED,
-	LAST_SIGNAL
-};
-
-static guint     signals[LAST_SIGNAL] = {0};
 static gpointer  parent_class = NULL;
 
 GType
@@ -138,34 +132,16 @@ account_class_init (GossipAccountClass *class)
 	object_class->get_property = account_get_property;
 	object_class->set_property = account_set_property;
 
-	signals[CHANGED] =
-		g_signal_new ("changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      libgossip_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
-
 	g_object_class_install_property (object_class,
 					 PROP_TYPE,
 					 g_param_spec_int ("type",
 							   "Account Type",
 							   "The account protocol type, e.g. MSN",
-							   G_MININT,
-							   G_MAXINT,
-							   GOSSIP_ACCOUNT_TYPE_JABBER,
+							   GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY,
+							   GOSSIP_ACCOUNT_TYPE_UNKNOWN,
+							   GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY,
 							   G_PARAM_READWRITE));
 
-	g_object_class_install_property (object_class,
-					 PROP_ID,
-					 g_param_spec_string ("id",
-							      "Account ID",
-							      "Your unique network ID, e.g. a JID",
-							      "Myself",
-							      G_PARAM_READWRITE));
-
 	g_object_class_install_property (object_class,
 					 PROP_NAME,
 					 g_param_spec_string ("name",
@@ -197,14 +173,10 @@ static void
 account_init (GossipAccount *account)
 {
 	GossipAccountPriv *priv;
-	static gint        id = 1;
 
 	priv = GET_PRIV (account);
 
-	priv->unique_id    = id++;
-
 	priv->type         = 0;
-	priv->id           = NULL;
 	priv->name         = NULL;
 	priv->auto_connect = TRUE;
 	priv->use_proxy    = FALSE;
@@ -212,7 +184,7 @@ account_init (GossipAccount *account)
 	priv->parameters = g_hash_table_new_full (g_str_hash,
 						  g_str_equal,
 						  g_free,
-						  (GDestroyNotify)account_param_free);
+						  (GDestroyNotify) account_param_free);
 }
 
 static void
@@ -222,8 +194,8 @@ account_finalize (GObject *object)
 
 	priv = GET_PRIV (object);
 
-	g_free (priv->id);
 	g_free (priv->name);
+
 	g_hash_table_unref (priv->parameters);
 
 	(G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -245,9 +217,6 @@ account_get_property (GObject    *object,
 	case PROP_TYPE:
 		g_value_set_int (value, priv->type);
 		break;
-	case PROP_ID:
-		g_value_set_string (value, priv->id);
-		break;
 	case PROP_NAME:
 		g_value_set_string (value, priv->name);
 		break;
@@ -278,10 +247,6 @@ account_set_property (GObject      *object,
 		account_set_type (GOSSIP_ACCOUNT (object),
 				  g_value_get_int (value));
 		break;
-	case PROP_ID:
-		gossip_account_set_id (GOSSIP_ACCOUNT (object),
-					 g_value_get_string (value));
-		break;
 	case PROP_NAME:
 		gossip_account_set_name (GOSSIP_ACCOUNT (object),
 					 g_value_get_string (value));
@@ -306,18 +271,24 @@ account_param_new_valist (GossipAccount *account,
 			  va_list        var_args)
 {
 	GossipAccountPriv *priv;
-	const gchar       *name;
+	GossipAccountType  account_type;
+	const gchar       *param_name;
 
 	priv = GET_PRIV (account);
 
-	for (name = first_param_name; name; name = va_arg (var_args, gchar*)) {
+	account_type = gossip_account_get_type (account);
+
+	for (param_name = first_param_name; 
+	     param_name; 
+	     param_name = va_arg (var_args, gchar*)) {
 		GossipAccountParam *param;
 		GType               g_type;
 		gchar              *error = NULL;
 
-		param = g_hash_table_lookup (priv->parameters, name);
+		param_name = account_param_name_convert (account_type, param_name);
+		param = g_hash_table_lookup (priv->parameters, param_name);
 		if (param) {
-			g_warning ("GossipAccount already has a parameter named `%s'", name);
+			g_warning ("GossipAccount already has a parameter named `%s'", param_name);
 			break;
 		}
 
@@ -336,7 +307,7 @@ account_param_new_valist (GossipAccount *account,
 		param->flags = va_arg (var_args, GossipAccountParamFlags);
 
 		g_hash_table_insert (priv->parameters,
-				     g_strdup (name),
+				     g_strdup (param_name),
 				     param);
 	}
 }
@@ -347,26 +318,40 @@ account_param_set_valist (GossipAccount *account,
 			  va_list        var_args)
 {
 	GossipAccountPriv *priv;
-	const gchar       *name;
+	GossipAccountType  account_type;
+	const gchar       *param_name;
 
 	priv = GET_PRIV (account);
 
-	for (name = first_param_name; name; name = va_arg (var_args, gchar*)) {
+	account_type = gossip_account_get_type (account);
+
+	for (param_name = first_param_name; 
+	     param_name; 
+	     param_name = va_arg (var_args, gchar*)) {
 		GossipAccountParam *param;
 		gchar              *error = NULL;
+		GValue              g_value = {0, };
 
-		param = g_hash_table_lookup (priv->parameters, name);
+		param_name = account_param_name_convert (account_type, param_name);
+		param = g_hash_table_lookup (priv->parameters, param_name);
 		if (!param) {
-			g_warning ("GossipAccount has no parameter named `%s'", name);
+			g_warning ("GossipAccount has no parameter named `%s'", param_name);
 			break;
 		}
 
-		G_VALUE_COLLECT (&param->g_value, var_args, 0, &error);
+		g_value_init (&g_value, G_VALUE_TYPE (&param->g_value));
+		G_VALUE_COLLECT (&g_value, var_args, 0, &error);
 		if (error) {
+			g_value_unset (&g_value);
 			g_warning ("%s: %s", G_STRFUNC, error);
 			g_free (error);
 			break;
 		}
+		if (!gossip_g_value_equal (&g_value, &param->g_value)) {
+			param->modified = TRUE;
+			g_value_copy (&g_value, &param->g_value);
+		}
+		g_value_unset (&g_value);
 	}
 }
 
@@ -376,14 +361,18 @@ account_param_get_valist (GossipAccount *account,
 			  va_list        var_args)
 {
 	GossipAccountPriv *priv;
+	GossipAccountType  account_type;
 	const gchar       *name;
 
 	priv = GET_PRIV (account);
 
+	account_type = gossip_account_get_type (account);
+
 	for (name = first_param_name; name; name = va_arg (var_args, gchar*)) {
 		GossipAccountParam *param;
 		gchar              *error = NULL;
 
+		name = account_param_name_convert (account_type, name);
 		param = g_hash_table_lookup (priv->parameters, name);
 		if (!param) {
 			g_warning ("GossipAccount has no parameter named `%s'", name);
@@ -400,7 +389,7 @@ account_param_get_valist (GossipAccount *account,
 }
 
 void
-gossip_account_param_new (GossipAccount *account,
+gossip_account_new_param (GossipAccount *account,
 			  const gchar   *first_param_name,
 			  ...)
 {
@@ -411,39 +400,43 @@ gossip_account_param_new (GossipAccount *account,
 	va_start (var_args, first_param_name);
 	account_param_new_valist (account, first_param_name, var_args);
 	va_end (var_args);
-
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
 void
-gossip_account_param_new_g_value (GossipAccount           *account,
+gossip_account_new_param_g_value (GossipAccount           *account,
 				  const gchar             *param_name,
 				  const GValue            *g_value,
 				  GossipAccountParamFlags  flags)
 {
 	GossipAccountParam *param;
 	GossipAccountPriv  *priv;
+	GossipAccountType   account_type;
+	const gchar        *param_name_converted;
 
 	priv = GET_PRIV (account);
 
-	param = g_hash_table_lookup (priv->parameters, param_name);
+	account_type = gossip_account_get_type (account);
+
+	param_name_converted = account_param_name_convert (account_type, param_name);
+	param = g_hash_table_lookup (priv->parameters, param_name_converted);
 	if (param) {
-		g_warning ("GossipAccount already has a parameter named `%s'", param_name);
+		g_warning ("GossipAccount already has a parameter named `%s'", 
+			   param_name_converted);
 		return;
 	}
-	param = g_new0 (GossipAccountParam, 1);
 
+	param = g_new0 (GossipAccountParam, 1);
 	g_value_init (&param->g_value, G_VALUE_TYPE (g_value));
 	g_value_copy (g_value, &param->g_value);
 	param->flags = flags;
 
 	g_hash_table_insert (priv->parameters,
-			     g_strdup (param_name),
+			     g_strdup (param_name_converted),
 			     param);
 }
 
 void
-gossip_account_param_set (GossipAccount *account,
+gossip_account_set_param (GossipAccount *account,
 			  const gchar   *first_param_name,
 			  ...)
 {
@@ -454,35 +447,38 @@ gossip_account_param_set (GossipAccount *account,
 	va_start (var_args, first_param_name);
 	account_param_set_valist (account, first_param_name, var_args);
 	va_end (var_args);
-
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
 void
-gossip_account_param_set_g_value (GossipAccount           *account,
-				  const gchar             *param_name,
-				  const GValue            *g_value)
+gossip_account_set_param_g_value (GossipAccount *account,
+				  const gchar   *param_name,
+				  const GValue  *g_value)
 {
 	GossipAccountPriv  *priv;
 	GossipAccountParam *param;
+	GossipAccountType   account_type;
 
 	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
 
 	priv = GET_PRIV (account);
 
+	account_type = gossip_account_get_type (account);
+
+	param_name = account_param_name_convert (account_type, param_name);
 	param = g_hash_table_lookup (priv->parameters, param_name);
 	if (!param) {
 		g_warning ("GossipAccount has no parameter named `%s'", param_name);
 		return;
 	}
 
-	g_value_copy (g_value, &param->g_value);
-
-	g_signal_emit (account, signals[CHANGED], 0);
+	if (!gossip_g_value_equal (g_value, &param->g_value)) {
+		param->modified = TRUE;
+		g_value_copy (g_value, &param->g_value);
+	}
 }
 
 void
-gossip_account_param_get (GossipAccount *account,
+gossip_account_get_param (GossipAccount *account,
 			  const gchar   *first_param_name,
 			  ...)
 {
@@ -496,16 +492,20 @@ gossip_account_param_get (GossipAccount *account,
 }
 
 const GValue *
-gossip_account_param_get_g_value (GossipAccount *account,
+gossip_account_get_param_g_value (GossipAccount *account,
 				  const gchar   *param_name)
 {
 	GossipAccountPriv  *priv;
 	GossipAccountParam *param;
+	GossipAccountType   account_type;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
 
 	priv = GET_PRIV (account);
 
+	account_type = gossip_account_get_type (account);
+	
+	param_name = account_param_name_convert (account_type, param_name);
 	param = g_hash_table_lookup (priv->parameters, param_name);
 	if (!param) {
 		g_warning ("GossipAccount has no parameter named `%s'", param_name);
@@ -519,48 +519,66 @@ gboolean
 gossip_account_has_param (GossipAccount *account,
 			  const gchar   *param_name)
 {
-	GossipAccountPriv      *priv;
+	GossipAccountPriv *priv;
+	GossipAccountType  account_type;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), FALSE);
 
 	priv = GET_PRIV (account);
 
+	account_type = gossip_account_get_type (account);
+	param_name = account_param_name_convert (account_type, param_name);
+
 	return g_hash_table_lookup (priv->parameters, param_name) != NULL;
 }
 
-void
-gossip_account_param_foreach (GossipAccount           *account,
-			      GossipAccountParamFunc   callback,
-			      gpointer                 user_data)
+static void
+account_param_get_all_foreach (gchar              *param_name,
+			       GossipAccountParam *param,
+			       GetParamsData      *data)
 {
-	GossipAccountPriv      *priv;
-	GossipAccountParamData *data;
+	const gchar *final_param_name;
 
-	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
+	final_param_name = account_param_name_convert (data->account_type, param_name);
+	data->params = g_list_prepend (data->params, g_strdup (final_param_name));
+}
+
+GList *
+gossip_account_get_param_all (GossipAccount *account)
+{
+	GossipAccountPriv *priv;
+	GetParamsData      data;
+
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
 
 	priv = GET_PRIV (account);
 
-	data = g_new0 (GossipAccountParamData, 1);
-	data->callback = callback;
-	data->user_data = user_data;
-	data->account = account;
+	data.account_type = gossip_account_get_type (account);
+	data.params = NULL;
 
 	g_hash_table_foreach (priv->parameters,
 			      (GHFunc) account_param_get_all_foreach,
-			      data);
+			      &data);
+
+	data.params = g_list_sort (data.params, (GCompareFunc) strcmp);
+
+	return data.params;
 }
 
-static void
-account_param_get_all_foreach (gchar                  *param_name,
-			       GossipAccountParam     *param,
-			       GossipAccountParamData *data)
+GossipAccountParam *
+gossip_account_get_param_param (GossipAccount *account,
+				const gchar   *param_name)
 {
-	data->callback (data->account,
-			param_name,
-			param,
-			data->user_data);
+	GossipAccountPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
+
+	priv = GET_PRIV (account);
+
+	return g_hash_table_lookup (priv->parameters, param_name);
 }
 
+
 static void
 account_param_free (GossipAccountParam *param)
 {
@@ -568,6 +586,23 @@ account_param_free (GossipAccountParam *param)
 	g_slice_free (GossipAccountParam, param);
 }
 
+const gchar *
+account_param_name_convert (GossipAccountType  type,
+			    const gchar       *param_name)
+{
+	if (type == GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY) {
+		return param_name;
+	} 
+
+#ifdef HAVE_TELEPATHY
+	if (strcmp (param_name, "use_ssl") == 0) {
+		return "old-ssl";
+	}
+#endif
+
+	return param_name;
+}
+
 GossipAccountType
 gossip_account_get_type (GossipAccount *account)
 {
@@ -579,17 +614,6 @@ gossip_account_get_type (GossipAccount *account)
 	return priv->type;
 }
 
-const gchar *
-gossip_account_get_id (GossipAccount *account)
-{
-	GossipAccountPriv *priv;
-
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
-
-	priv = GET_PRIV (account);
-	return priv->id;
-}
-
 const gchar *
 gossip_account_get_name (GossipAccount *account)
 {
@@ -633,26 +657,6 @@ account_set_type (GossipAccount     *account,
 
 	priv = GET_PRIV (account);
 	priv->type = type;
-
-	g_signal_emit (account, signals[CHANGED], 0);
-}
-
-void
-gossip_account_set_id (GossipAccount *account,
-		       const gchar   *id)
-{
-	GossipAccountPriv *priv;
-
-	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
-	g_return_if_fail (id != NULL);
-
-	priv = GET_PRIV (account);
-
-	g_free (priv->id);
-	priv->id = g_strdup (id);
-
-	g_object_notify (G_OBJECT (account), "id");
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
 void
@@ -670,7 +674,6 @@ gossip_account_set_name (GossipAccount *account,
 	priv->name = g_strdup (name);
 
 	g_object_notify (G_OBJECT (account), "name");
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
 void
@@ -685,7 +688,6 @@ gossip_account_set_auto_connect (GossipAccount *account,
 	priv->auto_connect = auto_connect;
 
 	g_object_notify (G_OBJECT (account), "auto_connect");
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
 void
@@ -700,48 +702,34 @@ gossip_account_set_use_proxy (GossipAccount *account,
 	priv->use_proxy = use_proxy;
 
 	g_object_notify (G_OBJECT (account), "use_proxy");
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
 guint
 gossip_account_hash (gconstpointer key)
 {
-	GossipAccountPriv *priv;
-
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (key), 0);
-
-	priv = GET_PRIV (key);
-
-	return g_int_hash (&priv->unique_id);
+	return g_direct_hash (key);
 }
 
 gboolean
 gossip_account_equal (gconstpointer a,
 		      gconstpointer b)
 {
-	GossipAccountPriv *priv1;
-	GossipAccountPriv *priv2;
-
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (a), FALSE);
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (b), FALSE);
-
-	priv1 = GET_PRIV (a);
-	priv2 = GET_PRIV (b);
-
-	return (priv1->unique_id == priv2->unique_id);
+	return g_direct_equal (a, b);
 }
 
 const gchar *
 gossip_account_type_to_string (GossipAccountType type)
 {
 	switch (type) {
-	case GOSSIP_ACCOUNT_TYPE_JABBER:  return "Jabber";
-	case GOSSIP_ACCOUNT_TYPE_AIM:     return "AIM";
-	case GOSSIP_ACCOUNT_TYPE_ICQ:     return "ICQ";
-	case GOSSIP_ACCOUNT_TYPE_MSN:     return "MSN";
-	case GOSSIP_ACCOUNT_TYPE_YAHOO:   return "Yahoo!";
-
-	case GOSSIP_ACCOUNT_TYPE_UNKNOWN:
+	case GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY:  return "Jabber";
+	case GOSSIP_ACCOUNT_TYPE_JABBER:         return "Experimental: Jabber";
+	case GOSSIP_ACCOUNT_TYPE_AIM:            return "Experimental: AIM";
+	case GOSSIP_ACCOUNT_TYPE_ICQ:            return "Experimental: ICQ";
+	case GOSSIP_ACCOUNT_TYPE_MSN:            return "Experimental: MSN";
+	case GOSSIP_ACCOUNT_TYPE_YAHOO:          return "Experimental: Yahoo!";
+	case GOSSIP_ACCOUNT_TYPE_IRC:            return "Experimental: IRC";
+	case GOSSIP_ACCOUNT_TYPE_SALUT:          return "Experimental: Salut";
+       	case GOSSIP_ACCOUNT_TYPE_UNKNOWN:
 	default:
 		break;
 	}
@@ -753,20 +741,30 @@ GossipAccountType
 gossip_account_string_to_type (const gchar *str)
 {
 	if (gossip_strncasecmp (str, "Jabber", -1) == 0) {
+		return GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY;
+	}
+	else if (gossip_strncasecmp (str, "Experimental: Jabber", -1) == 0) {
 		return GOSSIP_ACCOUNT_TYPE_JABBER;
 	}
-	else if (gossip_strncasecmp (str, "AIM", -1) == 0) {
+	else if (gossip_strncasecmp (str, "Experimental: AIM", -1) == 0) {
 		return GOSSIP_ACCOUNT_TYPE_AIM;
 	}
-	else if (gossip_strncasecmp (str, "ICQ", -1) == 0) {
+	else if (gossip_strncasecmp (str, "Experimental: ICQ", -1) == 0) {
 		return GOSSIP_ACCOUNT_TYPE_ICQ;
 	}
-	else if (gossip_strncasecmp (str, "MSN", -1) == 0) {
+	else if (gossip_strncasecmp (str, "Experimental: MSN", -1) == 0) {
 		return GOSSIP_ACCOUNT_TYPE_MSN;
 	}
-	else if (gossip_strncasecmp (str, "Yahoo!", -1) == 0) {
+	else if (gossip_strncasecmp (str, "Experimental: Yahoo!", -1) == 0) {
 		return GOSSIP_ACCOUNT_TYPE_YAHOO;
 	}
+	else if (gossip_strncasecmp (str, "Experimental: IRC", -1) == 0) {
+		return GOSSIP_ACCOUNT_TYPE_IRC;
+	} else if (gossip_strncasecmp (str, "Experimental: Salut", -1) == 0) {
+		return GOSSIP_ACCOUNT_TYPE_SALUT;
+	}
 
 	return GOSSIP_ACCOUNT_TYPE_UNKNOWN;
 }
+
+
