@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Martyn Russell <mr@gnome.org>
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,23 +16,36 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
+
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
-#include <libgossip/gossip-account.h>
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-protocol.h>
+#include <libgossip/gossip-session.h>
 #include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-account-manager.h>
 
+#ifdef HAVE_TELEPATHY
+#include <protocols/telepathy/gossip-telepathy-cmgr.h>
+#include "gossip-protocol-chooser.h"
+#endif
+
+#include "gossip-app.h"
+#include "gossip-account-widget-irc.h"
 #include "gossip-account-widget-jabber.h"
+#include "gossip-account-widget-msn.h"
 #include "gossip-account-widget-generic.h"
 #include "gossip-accounts-dialog.h"
-#include "gossip-app.h"
+#include "gossip-ui-utils.h"
 
 #define DEBUG_DOMAIN "AccountsDialog"
 
@@ -45,7 +58,7 @@ typedef struct {
 	GtkWidget *alignment_settings;
 
 	GtkWidget *vbox_details;
-	GtkWidget *hbox_no_account;
+	GtkWidget *frame_no_account;
 	GtkWidget *label_no_account;
 	GtkWidget *label_no_account_blurb;
 
@@ -54,8 +67,15 @@ typedef struct {
 	GtkWidget *button_remove;
 	GtkWidget *button_connect;
 
-	GtkWidget *label_name;
+	GtkWidget *frame_new_account;
+	GtkWidget *combobox_type;
 	GtkWidget *entry_name;
+	GtkWidget *button_create;
+	GtkWidget *button_back;
+
+	GtkWidget *image_type;
+	GtkWidget *label_name;
+	GtkWidget *label_type;
 	GtkWidget *checkbutton_connect;
 	GtkWidget *checkbutton_proxy;
 	GtkWidget *settings_widget;
@@ -71,90 +91,94 @@ typedef struct {
 	GtkComboBox   *combobox;
 } SetAccountData;
 
-static void           accounts_dialog_setup                      (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_set_connecting             (GossipAccountsDialog  *dialog,
-								  GossipAccount         *account);
-static void           accounts_dialog_update_connect_button      (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_update_account             (GossipAccountsDialog  *dialog,
-								  GossipAccount         *account);   
-static void           accounts_dialog_model_setup                (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_model_add_columns          (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_model_pixbuf_data_func     (GtkTreeViewColumn     *tree_column,
-								  GtkCellRenderer       *cell,
-								  GtkTreeModel          *model,
-								  GtkTreeIter           *iter,
-								  GossipAccountsDialog  *dialog);
-static GossipAccount *accounts_dialog_model_get_selected         (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_model_set_selected         (GossipAccountsDialog  *dialog,
-								  GossipAccount         *account);
-static void           accounts_dialog_model_select_first         (GossipAccountsDialog  *dialog);
-static gboolean       accounts_dialog_model_remove_selected      (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_model_selection_changed    (GtkTreeSelection      *selection,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_model_cell_edited          (GtkCellRendererText   *cell,
-								  const gchar           *path_string,
-								  const gchar           *new_text,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_account_added_cb           (GossipAccountManager  *manager,
-								  GossipAccount         *account,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_account_removed_cb         (GossipAccountManager  *manager,
-								  GossipAccount         *account,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_account_name_changed_cb    (GossipAccount         *account,
-								  GParamSpec            *param,
-								  GossipAccountsDialog  *dialog);
-static gboolean       accounts_dialog_flash_connecting_foreach   (GtkTreeModel          *model,
-								  GtkTreePath           *path,
-								  GtkTreeIter           *iter,
-								  gpointer               user_data);
-static gboolean       accounts_dialog_flash_connecting_cb        (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_protocol_connecting_cb     (GossipSession         *session,
-								  GossipAccount         *account,
-								  GossipProtocol        *protocol,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_protocol_connected_cb      (GossipSession         *session,
-								  GossipAccount         *account,
-								  GossipProtocol        *protocol,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_protocol_disconnected_cb   (GossipSession         *session,
-								  GossipAccount         *account,
-								  GossipProtocol        *protocol,
-								  gint                   reason,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_protocol_error_cb          (GossipSession         *session,
-								  GossipProtocol        *protocol,
-								  GossipAccount         *account,
-								  GError                *error,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_button_add_clicked_cb      (GtkWidget             *button,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_remove_response_cb         (GtkWidget             *dialog,
-								  gint                   response,
-								  GossipAccount         *account);
-static void           accounts_dialog_button_remove_clicked_cb   (GtkWidget             *button,
-								 GossipAccountsDialog   *dialog);
-static void           accounts_dialog_treeview_row_activated_cb  (GtkTreeView           *tree_view,
-								  GtkTreePath           *path,
-								  GtkTreeViewColumn     *column,
-								  gpointer              *dialog);
-static void           accounts_dialog_save                       (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_entry_changed_cb           (GtkWidget             *widget,
-								  GossipAccountsDialog  *dialog);
-static gboolean       accounts_dialog_entry_focus_cb             (GtkWidget             *widget,
-								  GdkEventFocus         *event,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_checkbutton_toggled_cb     (GtkWidget             *widget,
-								  GossipAccountsDialog  *dialog);
-static gboolean       accounts_dialog_foreach                    (GtkTreeModel          *model,
-								  GtkTreePath           *path,
-								  GtkTreeIter           *iter,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_response_cb                (GtkWidget             *widget,
-								  gint                   response,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_destroy_cb                 (GtkWidget             *widget,
-								  GossipAccountsDialog  *dialog);
+static void           accounts_dialog_setup                     (GossipAccountsDialog *dialog);
+static void           accounts_dialog_set_connecting            (GossipAccountsDialog *dialog,
+								 GossipAccount        *account);
+static void           accounts_dialog_update_connect_button     (GossipAccountsDialog *dialog);
+static void           accounts_dialog_update_account            (GossipAccountsDialog *dialog,
+								 GossipAccount        *account);
+static void           accounts_dialog_model_setup               (GossipAccountsDialog *dialog);
+static void           accounts_dialog_model_add_columns         (GossipAccountsDialog *dialog);
+static void           accounts_dialog_model_pixbuf_data_func    (GtkTreeViewColumn    *tree_column,
+								 GtkCellRenderer      *cell,
+								 GtkTreeModel         *model,
+								 GtkTreeIter          *iter,
+								 GossipAccountsDialog *dialog);
+static GossipAccount *accounts_dialog_model_get_selected        (GossipAccountsDialog *dialog);
+static void           accounts_dialog_model_set_selected        (GossipAccountsDialog *dialog,
+								 GossipAccount        *account);
+static void           accounts_dialog_model_select_first        (GossipAccountsDialog *dialog);
+static gboolean       accounts_dialog_model_remove_selected     (GossipAccountsDialog *dialog);
+static void           accounts_dialog_model_selection_changed   (GtkTreeSelection     *selection,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_model_cell_edited         (GtkCellRendererText  *cell,
+								 const gchar          *path_string,
+								 const gchar          *new_text,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_account_added_cb          (GossipAccountManager *manager,
+								 GossipAccount        *account,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_account_removed_cb        (GossipAccountManager *manager,
+								 GossipAccount        *account,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_account_name_changed_cb   (GossipAccount        *account,
+								 GParamSpec           *param,
+								 GossipAccountsDialog *dialog);
+static gboolean       accounts_dialog_flash_connecting_foreach  (GtkTreeModel         *model,
+								 GtkTreePath          *path,
+								 GtkTreeIter          *iter,
+								 gpointer              user_data);
+static gboolean       accounts_dialog_flash_connecting_cb       (GossipAccountsDialog *dialog);
+static void           accounts_dialog_protocol_connecting_cb    (GossipSession        *session,
+								 GossipAccount        *account,
+								 GossipProtocol       *protocol,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_protocol_connected_cb     (GossipSession        *session,
+								 GossipAccount        *account,
+								 GossipProtocol       *protocol,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_protocol_disconnected_cb  (GossipSession        *session,
+								 GossipAccount        *account,
+								 GossipProtocol       *protocol,
+								 gint                  reason,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_protocol_error_cb         (GossipSession        *session,
+								 GossipProtocol       *protocol,
+								 GossipAccount        *account,
+								 GError               *error,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_entry_name_changed_cb     (GtkWidget            *widget,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_button_create_clicked_cb  (GtkWidget            *button,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_button_back_clicked_cb    (GtkWidget            *button,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_button_connect_clicked_cb (GtkWidget            *button,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_button_add_clicked_cb     (GtkWidget            *button,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_remove_response_cb        (GtkWidget            *dialog,
+								 gint                  response,
+								 GossipAccount        *account);
+static void           accounts_dialog_button_remove_clicked_cb  (GtkWidget            *button,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_treeview_row_activated_cb (GtkTreeView          *tree_view,
+								 GtkTreePath          *path,
+								 GtkTreeViewColumn    *column,
+								 gpointer             *dialog);
+static void           accounts_dialog_save                      (GossipAccountsDialog *dialog);
+static void           accounts_dialog_checkbutton_toggled_cb    (GtkWidget            *widget,
+								 GossipAccountsDialog *dialog);
+static gboolean       accounts_dialog_foreach                   (GtkTreeModel         *model,
+								 GtkTreePath          *path,
+								 GtkTreeIter          *iter,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_response_cb               (GtkWidget            *widget,
+								 gint                  response,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_destroy_cb                (GtkWidget            *widget,
+								 GossipAccountsDialog *dialog);
+
 enum {
 	COL_NAME,
 	COL_EDITABLE,
@@ -356,7 +380,7 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 		GtkTreeView  *view;
 		GtkTreeModel *model;
 
-		gtk_widget_show (dialog->hbox_no_account);
+		gtk_widget_show (dialog->frame_no_account);
 		gtk_widget_hide (dialog->vbox_details);
 
 		gtk_widget_set_sensitive (dialog->button_connect, FALSE);
@@ -385,18 +409,32 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 						"for you to start configuring."));
 		}
 	} else {
-		gtk_widget_hide (dialog->hbox_no_account);
+		gtk_widget_hide (dialog->frame_no_account);
 		gtk_widget_show (dialog->vbox_details);
 
 		switch (gossip_account_get_type (account)) {
-		case GOSSIP_ACCOUNT_TYPE_JABBER:
-			dialog->settings_widget = gossip_account_widget_jabber_new 
-				(account, dialog->label_name);
+		case GOSSIP_ACCOUNT_TYPE_IRC:
+			dialog->settings_widget = 
+				gossip_account_widget_irc_new (account);
 			break;
+
+		case GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY:
+		case GOSSIP_ACCOUNT_TYPE_JABBER:
+			dialog->settings_widget = 
+				gossip_account_widget_jabber_new (account);
+			break;
+
+		case GOSSIP_ACCOUNT_TYPE_MSN:
+			dialog->settings_widget = 
+				gossip_account_widget_msn_new (account);
+			break;
+
 		default:
-			dialog->settings_widget = gossip_account_widget_generic_new 
-				(account, dialog->label_name);
+			dialog->settings_widget = 
+				gossip_account_widget_generic_new (account);
 		}
+		
+		gtk_widget_grab_focus (dialog->settings_widget);
 	}
 
 	if (dialog->settings_widget) {
@@ -405,7 +443,20 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 	}
 
 	if (account) {
-		gtk_entry_set_text (GTK_ENTRY (dialog->entry_name), 
+		GossipAccountType  account_type;
+		const gchar       *account_type_str;
+		GdkPixbuf         *pixbuf;
+
+		pixbuf = gossip_pixbuf_from_account (account, GTK_ICON_SIZE_DIALOG);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->image_type), pixbuf);
+		if (pixbuf) {
+			g_object_unref (pixbuf);
+		}
+
+		account_type = gossip_account_get_type (account);
+		account_type_str = gossip_account_type_to_string (account_type);
+		gtk_label_set_text (GTK_LABEL (dialog->label_type), account_type_str);
+		gtk_label_set_text (GTK_LABEL (dialog->label_name), 
 				    gossip_account_get_name (account));
 
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_proxy),
@@ -1034,6 +1085,99 @@ accounts_dialog_protocol_error_cb (GossipSession        *session,
 	accounts_dialog_update_connect_button (dialog);
 }
 
+static void          
+accounts_dialog_entry_name_changed_cb (GtkWidget             *widget,
+				       GossipAccountsDialog  *dialog)
+{
+	const gchar *str;
+	
+	str = gtk_entry_get_text (GTK_ENTRY (widget));
+	gtk_widget_set_sensitive (dialog->button_create, !G_STR_EMPTY (str));
+}
+
+static void
+accounts_dialog_button_create_clicked_cb (GtkWidget             *button,
+					  GossipAccountsDialog  *dialog)
+{
+	GossipSession        *session;
+	GossipAccountManager *manager;
+	GossipAccount        *account;
+	GossipAccountType     account_type;
+	gchar                *account_type_str;
+	const gchar          *str;
+
+	/* Update widgets */
+	gtk_widget_show (dialog->vbox_details);
+	gtk_widget_hide (dialog->frame_no_account);
+	gtk_widget_hide (dialog->frame_new_account);
+
+	/* Get account type */
+	account_type_str = gtk_combo_box_get_active_text (GTK_COMBO_BOX (dialog->combobox_type));
+	if (!account_type_str) {
+		return;
+	}
+
+	account_type = GOSSIP_ACCOUNT_TYPE_UNKNOWN;
+
+	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY);
+	if (strcmp (account_type_str, str) == 0) {
+		account_type = GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY;
+	} 
+
+#ifdef HAVE_TELEPATHY
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
+	if (strcmp (account_type_str, str) == 0) {
+		account_type = GOSSIP_ACCOUNT_TYPE_JABBER;
+	} 
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_MSN);
+	if (strcmp (account_type_str, str) == 0) {
+		account_type = GOSSIP_ACCOUNT_TYPE_MSN;
+	}
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_IRC);
+	if (strcmp (account_type_str, str) == 0) {
+		account_type = GOSSIP_ACCOUNT_TYPE_IRC;
+	}
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_SALUT);
+	if (strcmp (account_type_str, str) == 0) {
+		account_type = GOSSIP_ACCOUNT_TYPE_SALUT;
+	}
+#endif
+	
+	if (account_type == GOSSIP_ACCOUNT_TYPE_UNKNOWN) {
+		g_warning ("We have an unknown account type selected in the GtkComboBox");
+		return;
+	}
+
+	/* Create account */
+	session = gossip_app_get_session ();
+	manager = gossip_session_get_account_manager (session);
+	account = gossip_session_new_account (session, account_type);
+
+	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_name));
+	gossip_account_set_name (account, str);
+
+	gossip_account_manager_add (manager, account);
+	accounts_dialog_model_set_selected (dialog, account);
+	g_object_unref (account);
+}
+
+static void
+accounts_dialog_button_back_clicked_cb (GtkWidget             *button,
+					GossipAccountsDialog  *dialog)
+{
+	GossipAccount *account;
+
+	gtk_widget_hide (dialog->vbox_details);
+	gtk_widget_hide (dialog->frame_no_account);
+	gtk_widget_hide (dialog->frame_new_account);
+
+	account = accounts_dialog_model_get_selected (dialog);
+	accounts_dialog_update_account (dialog, account);
+}
+
 static void
 accounts_dialog_button_connect_clicked_cb (GtkWidget            *button,
 					   GossipAccountsDialog *dialog)
@@ -1094,18 +1238,12 @@ static void
 accounts_dialog_button_add_clicked_cb (GtkWidget            *button,
 				       GossipAccountsDialog *dialog)
 {
-	GossipAccountManager *manager;
-	GossipAccount        *account;
-	GossipSession        *session;
-
-	session = gossip_app_get_session ();
-	manager = gossip_session_get_account_manager (session);
-	account = gossip_session_new_account (session, GOSSIP_ACCOUNT_TYPE_JABBER);
-
-	gossip_account_manager_add (manager, account);
-	accounts_dialog_model_set_selected (dialog, account);
-	g_object_unref (account);
+	gtk_widget_hide (dialog->vbox_details);
+	gtk_widget_hide (dialog->frame_no_account);
+	gtk_widget_show (dialog->frame_new_account);
 
+	gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->combobox_type), 0);
+	gtk_entry_set_text (GTK_ENTRY (dialog->entry_name), "");
 	gtk_widget_grab_focus (dialog->entry_name);
 }
 
@@ -1143,7 +1281,7 @@ accounts_dialog_button_remove_clicked_cb (GtkWidget            *button,
 		(GTK_WINDOW (dialog->window),
 		 GTK_DIALOG_DESTROY_WITH_PARENT,
 		 GTK_MESSAGE_QUESTION,
-		 GTK_BUTTONS_YES_NO,
+		 GTK_BUTTONS_NONE,
 		 _("You are about to remove your %s account!\n"
 		   "Are you sure you want to proceed?"),
 		 gossip_account_get_name (account));
@@ -1156,6 +1294,13 @@ accounts_dialog_button_remove_clicked_cb (GtkWidget            *button,
 		   "Should you decide to add the account back at a later time, "
 		   "they will still be available."));
 
+	gtk_dialog_add_button (GTK_DIALOG (message_dialog),
+			       GTK_STOCK_CANCEL, 
+			       GTK_RESPONSE_NO);
+	gtk_dialog_add_button (GTK_DIALOG (message_dialog),
+			       GTK_STOCK_REMOVE, 
+			       GTK_RESPONSE_YES);
+
 	g_signal_connect (message_dialog, "response",
 			  G_CALLBACK (accounts_dialog_remove_response_cb),
 			  account);
@@ -1181,7 +1326,6 @@ accounts_dialog_save (GossipAccountsDialog *dialog)
 	GossipSession        *session;
 	GossipAccount        *account;
 	GossipAccountManager *manager;
- 	const gchar          *str;
 	gboolean              bool;
 
 	dialog->account_changed = FALSE;
@@ -1194,9 +1338,6 @@ accounts_dialog_save (GossipAccountsDialog *dialog)
 		return;
 	}
 
-	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_name));
- 	gossip_account_set_name (account, str); 
-
 	bool = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_connect));
 	gossip_account_set_auto_connect (account, bool);
 
@@ -1208,40 +1349,6 @@ accounts_dialog_save (GossipAccountsDialog *dialog)
 	g_object_unref (account);
 }
 
-static void
-accounts_dialog_entry_changed_cb (GtkWidget            *widget,
-				  GossipAccountsDialog *dialog)
-{
-	dialog->account_changed = TRUE;
-}
-
-static gboolean
-accounts_dialog_entry_focus_cb (GtkWidget            *widget,
-				GdkEventFocus        *event,
-				GossipAccountsDialog *dialog)
-{
-	GossipAccount *account;
-	const gchar   *str;
-
- 	account = accounts_dialog_model_get_selected (dialog);
-
-	str = gtk_entry_get_text (GTK_ENTRY (widget));
-	if (G_STR_EMPTY (str)) {
-		str = gossip_account_get_name (account);
-
-		gtk_entry_set_text (GTK_ENTRY (widget), str);
-		dialog->account_changed = FALSE;
-	}
-
-	if (dialog->account_changed) {
- 		accounts_dialog_save (dialog); 
-	}
-
-	g_object_unref (account);
-
-	return FALSE;
-}
-
 static void  
 accounts_dialog_checkbutton_toggled_cb (GtkWidget            *widget,
 					GossipAccountsDialog *dialog)
@@ -1334,7 +1441,9 @@ gossip_accounts_dialog_show (GossipAccount *account)
 	GossipAccountManager        *manager;
 	static GossipAccountsDialog *dialog = NULL;
 	GladeXML                    *glade;
-	GtkWidget                   *bbox, *button_close;
+	GtkWidget                   *bbox;
+	GtkWidget                   *button_close;
+	const gchar                 *str;
 
 	if (dialog) {
 		gtk_window_present (GTK_WINDOW (dialog->window));
@@ -1348,39 +1457,65 @@ gossip_accounts_dialog_show (GossipAccount *account)
 				       NULL,
 				       "accounts_dialog", &dialog->window,
 				       "vbox_details", &dialog->vbox_details,
-				       "hbox_no_account", &dialog->hbox_no_account,
+				       "frame_no_account", &dialog->frame_no_account,
 				       "label_no_account", &dialog->label_no_account,
 				       "label_no_account_blurb", &dialog->label_no_account_blurb,
 				       "alignment_settings", &dialog->alignment_settings,
 				       "dialog-action_area", &bbox,
 				       "treeview", &dialog->treeview,
+				       "frame_new_account", &dialog->frame_new_account,
+				       "combobox_type", &dialog->combobox_type,
+				       "entry_name", &dialog->entry_name,
+				       "button_create", &dialog->button_create,
+				       "button_back", &dialog->button_back,
+				       "image_type", &dialog->image_type,
+				       "label_type", &dialog->label_type,
+				       "label_name", &dialog->label_name,
+				       "checkbutton_connect", &dialog->checkbutton_connect,
+				       "checkbutton_proxy", &dialog->checkbutton_proxy,
 				       "button_remove", &dialog->button_remove,
 				       "button_connect", &dialog->button_connect,
 				       "button_close", &button_close,
-				       "label_name", &dialog->label_name,
-				       "entry_name", &dialog->entry_name,
-				       "checkbutton_connect", &dialog->checkbutton_connect,
-				       "checkbutton_proxy", &dialog->checkbutton_proxy,
 				       NULL);
 
 	gossip_glade_connect (glade,
 			      dialog,
 			      "accounts_dialog", "destroy", accounts_dialog_destroy_cb,
 			      "accounts_dialog", "response", accounts_dialog_response_cb,
+			      "button_create", "clicked", accounts_dialog_button_create_clicked_cb,
+			      "button_back", "clicked", accounts_dialog_button_back_clicked_cb,
+			      "entry_name", "changed", accounts_dialog_entry_name_changed_cb,
+			      "treeview", "row-activated", accounts_dialog_treeview_row_activated_cb,
+			      "checkbutton_connect", "toggled", accounts_dialog_checkbutton_toggled_cb,
+			      "checkbutton_proxy", "toggled", accounts_dialog_checkbutton_toggled_cb,
 			      "button_connect", "clicked", accounts_dialog_button_connect_clicked_cb,
 			      "button_add", "clicked", accounts_dialog_button_add_clicked_cb,
 			      "button_remove", "clicked", accounts_dialog_button_remove_clicked_cb,
-			      "treeview", "row-activated", accounts_dialog_treeview_row_activated_cb,
-			      "entry_name", "changed", accounts_dialog_entry_changed_cb,
-			      "entry_name", "focus-out-event", accounts_dialog_entry_focus_cb,
-			      "checkbutton_connect", "toggled", accounts_dialog_checkbutton_toggled_cb,
-			      "checkbutton_proxy", "toggled", accounts_dialog_checkbutton_toggled_cb,
 			      NULL);
 
 	g_object_add_weak_pointer (G_OBJECT (dialog->window), (gpointer) &dialog);
 
 	g_object_unref (glade);
 
+	/* Set up combo */
+	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
+
+#ifdef HAVE_TELEPATHY
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_MSN);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_IRC);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_SALUT);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
+#endif
+
+	/* Set up signalling */
 	session = gossip_app_get_session ();
 	manager = gossip_session_get_account_manager (session);
 
@@ -1416,7 +1551,7 @@ gossip_accounts_dialog_show (GossipAccount *account)
 	gtk_widget_show (dialog->window);
 
 	if (GOSSIP_IS_ACCOUNT (account)) {
-		/* if account was specified then we select it */
+		/* If account was specified then we select it */
 		accounts_dialog_model_set_selected (dialog, account);
 	} else {
 		accounts_dialog_model_select_first (dialog);
