@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2006 Imendio AB
+ * Copyright (C) 2006-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,20 +16,24 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Richard Hult <richard@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdk.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
+#ifdef HAVE_GNOME
+#include <gdk/gdkx.h>
+#endif
+
 #include "gossip-avatar-image.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
-		       GOSSIP_TYPE_AVATAR_IMAGE, GossipAvatarImagePriv))
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_AVATAR_IMAGE, GossipAvatarImagePriv))
 
 #define MAX_SMALL  48
 #define MAX_LARGE 400
@@ -37,7 +41,7 @@
 typedef struct {
 	GtkWidget   *image;
 	GtkWidget   *popup;
-	GtkTooltips *tooltips;	
+	GtkTooltips *tooltips;
 	GdkPixbuf   *pixbuf;
 } GossipAvatarImagePriv;
 
@@ -71,9 +75,9 @@ gossip_avatar_image_init (GossipAvatarImage *avatar_image)
 	GossipAvatarImagePriv *priv;
 
 	priv = GET_PRIV (avatar_image);
-	
+
 	priv->image = gtk_image_new ();
-	
+
 	gtk_container_add (GTK_CONTAINER (avatar_image), priv->image);
 
 	/* FIXME: Should just override the methods instead. */
@@ -107,16 +111,17 @@ avatar_image_finalize (GObject *object)
 	if (priv->popup) {
 		gtk_widget_destroy (priv->popup);
 	}
-	
+
 	if (priv->pixbuf) {
 		g_object_unref (priv->pixbuf);
 	}
 
 	g_object_unref (priv->tooltips);
-	
+
 	G_OBJECT_CLASS (gossip_avatar_image_parent_class)->finalize (object);
 }
 
+#ifdef HAVE_GNOME
 static GdkFilterReturn
 avatar_image_filter_func (GdkXEvent  *gdkxevent,
 			  GdkEvent   *event,
@@ -127,7 +132,7 @@ avatar_image_filter_func (GdkXEvent  *gdkxevent,
 	GossipAvatarImagePriv *priv;
 
 	priv = GET_PRIV (data);
-	
+
 	switch (xevent->type) {
 	case PropertyNotify:
 		atom = gdk_x11_get_xatom_by_name ("_NET_CURRENT_DESKTOP");
@@ -136,43 +141,48 @@ avatar_image_filter_func (GdkXEvent  *gdkxevent,
 				gtk_widget_destroy (priv->popup);
 				priv->popup = NULL;
 			}
-		}		
+		}
 		break;
 	}
-	
+
 	return GDK_FILTER_CONTINUE;
 }
+#endif
 
 static void
 avatar_image_add_filter (GossipAvatarImage *avatar_image)
 {
+#ifdef HAVE_GNOME
 	Window     window;
 	GdkWindow *gdkwindow;
 	gint       mask;
 
 	mask = PropertyChangeMask;
-	
+
 	window = GDK_ROOT_WINDOW ();
 	gdkwindow = gdk_xid_table_lookup (window);
-	
+
 	gdk_error_trap_push ();
 	if (gdkwindow) {
 		XWindowAttributes attrs;
 		XGetWindowAttributes (gdk_display, window, &attrs);
 		mask |= attrs.your_event_mask;
 	}
-	
+
 	XSelectInput (gdk_display, window, mask);
-	
+
 	gdk_error_trap_pop ();
-	
+
 	gdk_window_add_filter (NULL, avatar_image_filter_func, avatar_image);
+#endif
 }
 
 static void
 avatar_image_remove_filter (GossipAvatarImage *avatar_image)
 {
+#ifdef HAVE_GNOME
 	gdk_window_remove_filter (NULL, avatar_image_filter_func, avatar_image);
+#endif
 }
 
 static GdkPixbuf *
@@ -186,10 +196,10 @@ avatar_image_scale_down_if_necessary (GdkPixbuf *pixbuf, gint max_size)
 
 	if (width > max_size || height > max_size) {
 		factor = (gdouble) max_size / MAX (width, height);
-		
+
 		width = width * factor;
 		height = height * factor;
-		
+
 		return gdk_pixbuf_scale_simple (pixbuf,
 						width, height,
 						GDK_INTERP_HYPER);
@@ -213,19 +223,19 @@ avatar_image_button_press_event_cb (GtkWidget         *widget,
 	GdkPixbuf             *pixbuf;
 
 	priv = GET_PRIV (avatar_image);
-	
+
 	if (priv->popup) {
 		gtk_widget_destroy (priv->popup);
 		priv->popup = NULL;
 	}
-	
+
 	if (event->button != 1 || event->type != GDK_BUTTON_PRESS) {
 		return FALSE;
 	}
 
 	popup_width = gdk_pixbuf_get_width (priv->pixbuf);
 	popup_height = gdk_pixbuf_get_height (priv->pixbuf);
-	
+
 	width = priv->image->allocation.width;
 	height = priv->image->allocation.height;
 
@@ -239,29 +249,29 @@ avatar_image_button_press_event_cb (GtkWidget         *widget,
 	pixbuf = avatar_image_scale_down_if_necessary (priv->pixbuf, MAX_LARGE);
 	popup_width = gdk_pixbuf_get_width (pixbuf);
 	popup_height = gdk_pixbuf_get_height (pixbuf);
-	
+
 	popup = gtk_window_new (GTK_WINDOW_POPUP);
 
 	frame = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
 
 	gtk_container_add (GTK_CONTAINER (popup), frame);
-	
+
 	image = gtk_image_new ();
 	gtk_container_add (GTK_CONTAINER (frame), image);
 
 	gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
 	g_object_unref (pixbuf);
-	
+
 	gdk_window_get_origin (priv->image->window, &x, &y);
 
 	x = x - (popup_width - width) / 2;
 	y = y - (popup_height - height) / 2;
-	
+
 	gtk_window_move (GTK_WINDOW (popup), x, y);
 
 	priv->popup = popup;
-	
+
 	gtk_widget_show_all (popup);
 
 	return TRUE;
@@ -275,7 +285,7 @@ avatar_image_button_release_event_cb (GtkWidget         *widget,
 	GossipAvatarImagePriv *priv;
 
 	priv = GET_PRIV (avatar_image);
-		
+
 	if (event->button != 1 || event->type != GDK_BUTTON_RELEASE) {
 		return FALSE;
 	}
@@ -283,10 +293,10 @@ avatar_image_button_release_event_cb (GtkWidget         *widget,
 	if (!priv->popup) {
 		return TRUE;
 	}
-	
+
 	gtk_widget_destroy (priv->popup);
 	priv->popup = NULL;
-		
+
 	return TRUE;
 }
 
@@ -296,7 +306,7 @@ gossip_avatar_image_new (GdkPixbuf *pixbuf)
 	GossipAvatarImage *avatar_image;
 
 	avatar_image = g_object_new (GOSSIP_TYPE_AVATAR_IMAGE, NULL);
-	
+
 	gossip_avatar_image_set_pixbuf (avatar_image, pixbuf);
 
 	return GTK_WIDGET (avatar_image);
@@ -308,14 +318,14 @@ gossip_avatar_image_set_pixbuf (GossipAvatarImage *avatar_image,
 {
 	GossipAvatarImagePriv *priv;
 	GdkPixbuf             *scaled_pixbuf;
-	
+
 	priv = GET_PRIV (avatar_image);
 
 	if (priv->pixbuf) {
 		g_object_unref (priv->pixbuf);
 		priv->pixbuf = NULL;
 	}
-	
+
 	if (!pixbuf) {
 		gtk_widget_hide (priv->image);
 		return;
@@ -336,7 +346,7 @@ gossip_avatar_image_set_pixbuf (GossipAvatarImage *avatar_image,
 				      GTK_WIDGET (avatar_image),
 				      NULL, NULL);
 	}
-	
+
 	g_object_unref (scaled_pixbuf);
 
 	gtk_widget_show (priv->image);
