@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2006 Imendio AB
- * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <geertjan@gnome.org>
+ * Copyright (C) 2003-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,19 +16,28 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
+ *          Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
+
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
+#include <libgossip/gossip-contact.h>
 #include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-chatroom.h>
+#include <libgossip/gossip-message.h>
+#include <libgossip/gossip-chatroom-manager.h>
 #include <libgossip/gossip-conf.h>
-#include <libgossip/gossip-log.h>
-#include <libgossip/gossip-protocol.h>
 
 #include "gossip-add-contact-dialog.h"
 #include "gossip-app.h"
@@ -57,7 +65,6 @@ struct _GossipChatWindowPriv {
 
 	GossipChat  *current_chat;
 
-	gboolean     new_msg;
 	gboolean     page_added;
 	gboolean     dnd_same_window;
 
@@ -570,8 +577,6 @@ chat_window_avatar_changed_cb (GossipContact    *contact,
 	if (!gossip_contact_equal (current_contact, contact)) {
 		return;
 	}
-
-	chat_window_update_title (window, priv->current_chat);
 }
 
 static void
@@ -584,6 +589,19 @@ chat_window_close_clicked_cb (GtkWidget  *button,
 	gossip_chat_window_remove_chat (window, chat);
 }
 
+static void
+chat_window_close_button_style_set_cb (GtkWidget *button,
+				       GtkStyle  *previous_style,
+				       gpointer   user_data)
+{
+	gint h, w;
+
+	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (button),
+					   GTK_ICON_SIZE_MENU, &w, &h);
+
+	gtk_widget_set_size_request (button, w, h);
+}
+
 static GtkWidget *
 chat_window_create_label (GossipChatWindow *window,
 			  GossipChat       *chat)
@@ -594,12 +612,10 @@ chat_window_create_label (GossipChatWindow *window,
 	GtkWidget            *status_image;
 	GtkWidget            *close_button;
 	GtkWidget            *close_image;
-	const gchar          *name;
 	GtkWidget            *event_box;
 	GtkWidget            *event_box_hbox;
 	PangoAttrList        *attr_list;
 	PangoAttribute       *attr;
-	GtkRequisition        size;
 
 	priv = GET_PRIV (window);
 
@@ -609,10 +625,8 @@ chat_window_create_label (GossipChatWindow *window,
 	event_box = gtk_event_box_new ();
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (event_box), FALSE);
 
-	name = gossip_chat_get_name (chat);
-	name_label = gtk_label_new (name);
- 	gtk_label_set_ellipsize (GTK_LABEL (name_label), PANGO_ELLIPSIZE_END);
-
+	name_label = gtk_label_new (gossip_chat_get_name (chat));
+	gtk_label_set_ellipsize (GTK_LABEL (name_label), PANGO_ELLIPSIZE_END);
 
 	attr_list = pango_attr_list_new ();
 	attr = pango_attr_scale_new (1/1.2);
@@ -637,8 +651,6 @@ chat_window_create_label (GossipChatWindow *window,
 	g_object_set_data (G_OBJECT (chat), "chat-window-tab-image", status_image);
 	g_object_set_data (G_OBJECT (chat), "chat-window-tab-tooltip-widget", event_box);
 
-	chat_window_update_tooltip (window, chat);
-
 	close_button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
 
@@ -653,16 +665,20 @@ chat_window_create_label (GossipChatWindow *window,
 
 	close_image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
 
-	gtk_widget_size_request (close_image, &size);
-	gtk_widget_set_size_request (close_button, size.width, size.height);
-
 	gtk_container_add (GTK_CONTAINER (close_button), close_image);
-	gtk_container_set_border_width (GTK_CONTAINER (close_button), 0);
 
 	gtk_container_add (GTK_CONTAINER (event_box), event_box_hbox);
 	gtk_box_pack_start (GTK_BOX (hbox), event_box, TRUE, TRUE, 0);
 	gtk_box_pack_end (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
 
+	/* React to theme changes and also used to setup the initial size
+	 * correctly.
+	 */
+	g_signal_connect (close_button,
+			  "style-set",
+			  G_CALLBACK (chat_window_close_button_style_set_cb),
+			  chat);
+
 	g_signal_connect (close_button,
 			  "clicked",
 			  G_CALLBACK (chat_window_close_clicked_cb),
@@ -688,7 +704,8 @@ chat_window_update_status (GossipChatWindow *window,
 	gtk_image_set_from_pixbuf (image, pixbuf);
 
 	g_object_unref (pixbuf);
-
+	
+	chat_window_update_title (window, chat);
 	chat_window_update_tooltip (window, chat);
 }
 
@@ -696,39 +713,71 @@ static void
 chat_window_update_title (GossipChatWindow *window,
 			  GossipChat       *chat)
 {
-	GossipChatWindowPriv *priv;
-	GossipContact        *contact;
-	const gchar          *name;
-	GdkPixbuf 	     *pixbuf = NULL;
-
+	GossipChatWindowPriv	*priv;
+	GdkPixbuf		*pixbuf = NULL;
+	const gchar             *str;
+	gchar			*title;
+	gint  			 n_chats;
+	
 	priv = GET_PRIV (window);
-
-	/* Use the name from the supplied chat, which happens when we get a
-	 * message from an tab other than the active one. This makes it easier
-	 * to see who talked to you when you have multiple tabs.
-	 */
-	if (chat) {
-		name = gossip_chat_get_name (chat);
+	
+	n_chats = g_list_length (priv->chats);
+	if (n_chats == 1) {
+		if (priv->chats_new_msg) {
+			title = g_strdup_printf (
+				"%s - %s",
+				gossip_chat_get_name (priv->current_chat),
+				_("New Message"));
+		}
+		else if (gossip_chat_is_group_chat (priv->current_chat)) {
+			title = g_strdup_printf (
+				"%s - %s", 
+				gossip_chat_get_name (priv->current_chat),
+				_("Chat Room"));
+		} else {
+			title = g_strdup_printf (
+				"%s - %s", 
+				gossip_chat_get_name (priv->current_chat),
+				_("Conversation"));
+		}
 	} else {
-		name = gossip_chat_get_name (priv->current_chat);
+		if (priv->chats_new_msg) {
+			GString *names;
+			GList   *l;
+			gint     n_messages = 0;
+
+			names = g_string_new (NULL);
+
+			for (l = priv->chats_new_msg; l; l = l->next) {
+				n_messages++;
+				g_string_append (names,
+						 gossip_chat_get_name (l->data));
+				if (l->next) {
+					g_string_append (names, ", ");
+				}
+			}
+			
+			str = ngettext ("New Message", "New Messages", n_messages);
+			title = g_strdup_printf ("%s - %s", names->str, str);
+			g_string_free (names, TRUE);
+		} else {
+			str = ngettext ("Conversation", "Conversations (%d)", n_chats);
+			title = g_strdup_printf (str, n_chats);
+		}
 	}
 
-	gtk_window_set_title (GTK_WINDOW (priv->dialog), name);
+	gtk_window_set_title (GTK_WINDOW (priv->dialog), title);
+	g_free (title);
 
-	if (priv->new_msg) {
+	if (priv->chats_new_msg) {
 		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE,
 						   GTK_ICON_SIZE_MENU);
 	} else {
-		/* Update the avatar if we have one */
-		contact = gossip_chat_get_contact (priv->current_chat);
-		if (contact) {
-			pixbuf = gossip_pixbuf_avatar_from_contact (contact);
-		}
-
-		chat_window_set_urgency_hint (window, FALSE);
+		pixbuf = NULL;
 	}
 
 	gtk_window_set_icon (GTK_WINDOW (priv->dialog), pixbuf);
+
 	if (pixbuf) {
 		g_object_unref (pixbuf);
 	}
@@ -1205,7 +1254,6 @@ chat_window_tabs_left_activate_cb (GtkWidget        *menuitem,
 				    gossip_chat_get_widget (chat),
 				    index - 1);
 
-	chat_window_update_title (window, NULL);
 	chat_window_update_menu (window);
 	chat_window_update_status (window, chat);
 }
@@ -1227,7 +1275,6 @@ chat_window_tabs_right_activate_cb (GtkWidget        *menuitem,
 				    gossip_chat_get_widget (chat),
 				    index + 1);
 
-	chat_window_update_title (window, NULL);
 	chat_window_update_menu (window);
 	chat_window_update_status (window, chat);
 }
@@ -1279,7 +1326,6 @@ static void
 chat_window_status_changed_cb (GossipChat       *chat,
 			       GossipChatWindow *window)
 {
-	chat_window_update_title (window, NULL);
 	chat_window_update_menu (window);
 	chat_window_update_status (window, chat);
 }
@@ -1349,35 +1395,36 @@ chat_window_new_message_cb (GossipChat       *chat,
 			    GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
+	GossipContact        *own_contact;
+	gboolean              has_focus;
+	gboolean              needs_urgency;
 
 	priv = GET_PRIV (window);
 
-	gossip_request_user_attention ();
+	has_focus = gossip_chat_window_has_focus (window);
+	
+	if (has_focus && priv->current_chat == chat) {
+		gossip_debug (DEBUG_DOMAIN, "New message, we have focus");
+		return;
+	}
+	
+	gossip_debug (DEBUG_DOMAIN, "New message, no focus");
 
-	if (!gossip_chat_window_has_focus (window)) {
-		GossipContact *own_contact;
-
-		gossip_debug (DEBUG_DOMAIN, "New message, no focus");
-
-		priv->new_msg = TRUE;
-		chat_window_update_title (window, chat);
-
-		if (gossip_chat_is_group_chat (chat)) {
-			own_contact = gossip_chat_get_own_contact (chat);
-
-			if (gossip_chat_should_highlight_nick (message, own_contact)) {
-				gossip_debug (DEBUG_DOMAIN, "Highlight this nick");
-				chat_window_set_urgency_hint (window, TRUE);
-			}
-		} else {
-			chat_window_set_urgency_hint (window, TRUE);
+	needs_urgency = FALSE;
+	if (gossip_chat_is_group_chat (chat)) {
+		own_contact = gossip_chat_get_own_contact (chat);
+		
+		if (gossip_chat_should_highlight_nick (message, own_contact)) {
+			gossip_debug (DEBUG_DOMAIN, "Highlight this nick");
+			needs_urgency = TRUE;
 		}
 	} else {
-		gossip_debug (DEBUG_DOMAIN, "New message, we have focus");
+		needs_urgency = TRUE;
 	}
 
-	if (chat == priv->current_chat) {
-		return;
+	if (needs_urgency && !has_focus) {
+		chat_window_set_urgency_hint (window, TRUE);
+		gossip_request_user_attention ();
 	}
 
 	if (!g_list_find (priv->chats_new_msg, chat)) {
@@ -1436,9 +1483,9 @@ chat_window_detach_hook (GtkNotebook *source,
 }
 
 static void
-chat_window_page_switched_cb (GtkNotebook	     *notebook,
+chat_window_page_switched_cb (GtkNotebook      *notebook,
 			      GtkNotebookPage  *page,
-			      gint	      page_num,
+			      gint	        page_num,
 			      GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
@@ -1455,14 +1502,14 @@ chat_window_page_switched_cb (GtkNotebook	     *notebook,
 	if (priv->page_added) {
 		priv->page_added = FALSE;
 		gossip_chat_scroll_down (chat);
-	} else if (priv->current_chat == chat) {
+	}
+	else if (priv->current_chat == chat) {
 		return;
 	}
 
 	priv->current_chat = chat;
 	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
 
-	chat_window_update_title (window, NULL);
 	chat_window_update_menu (window);
 	chat_window_update_status (window, chat);
 }
@@ -1601,6 +1648,7 @@ chat_window_page_removed_cb (GtkNotebook      *notebook,
 		g_object_unref (window);
 	} else {
 		chat_window_update_menu (window);
+		chat_window_update_title (window, NULL);
 	}
 }
 
@@ -1614,10 +1662,13 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 	gossip_debug (DEBUG_DOMAIN, "Focus in event, updating title");
 
 	priv = GET_PRIV (window);
-	priv->new_msg = FALSE;
 
-	/* Set the title, since we now mark all unread messages as read */
-	chat_window_update_title (window, NULL);
+	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, priv->current_chat);
+
+	chat_window_set_urgency_hint (window, FALSE);
+	
+	/* Update the title, since we now mark all unread messages as read. */
+	chat_window_update_status (window, priv->current_chat);
 
 	return FALSE;
 }
@@ -1675,7 +1726,8 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		 * anyway with add_chat() and remove_chat().
 		 */
 		gtk_drag_finish (context, TRUE, FALSE, time);
-	} else if (info == DND_DRAG_TYPE_TAB) {
+	}
+	else if (info == DND_DRAG_TYPE_TAB) {
 		GossipChat        *chat = NULL;
 		GossipChatWindow  *old_window;
 		GtkWidget        **child = NULL;
