@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Martyn Russell <mr@gnome.org>
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,44 +16,44 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs.h>
+
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
-#include "libgossip/gossip-utils.h"
+#include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-utils.h>
+
 #include "gossip-contact-groups.h"
 
-#define DEBUG_MSG(x) 
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
+#define DEBUG_DOMAIN "ContactGroups"
 
 #define CONTACT_GROUPS_XML_FILENAME "contact-groups.xml"
 #define CONTACT_GROUPS_DTD_FILENAME "gossip-contact-groups.dtd"
 
-
 typedef struct {
 	gchar    *name;
 	gboolean  expanded;
 } ContactGroup;
 
+static void          contact_groups_file_parse (const gchar  *filename);
+static gboolean      contact_groups_file_save  (void);
+static ContactGroup *contact_group_new         (const gchar  *name,
+						gboolean      expanded);
+static void          contact_group_free        (ContactGroup *group);
 
-static void          contact_groups_file_parse    (const gchar  *filename);
-static gboolean      contact_groups_file_save     (void);
-
-static ContactGroup *contact_group_new            (const gchar  *name,
-						   gboolean      expanded);
-static void          contact_group_free           (ContactGroup *group);
-
-
-static GList *groups = NULL; 
-
+static GList *groups = NULL;
 
 void
 gossip_contact_groups_get_all (void)
@@ -69,35 +69,31 @@ gossip_contact_groups_get_all (void)
 	}
 
 	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
-	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
-		g_mkdir_with_parents (dir, S_IRUSR | S_IWUSR | S_IXUSR);
-	}
-
 	file_with_path = g_build_filename (dir, CONTACT_GROUPS_XML_FILENAME, NULL);
 	g_free (dir);
 
-	if (g_file_test(file_with_path, G_FILE_TEST_EXISTS)) {
+	if (g_file_test (file_with_path, G_FILE_TEST_EXISTS)) {
 		contact_groups_file_parse (file_with_path);
 	}
-	
+
 	g_free (file_with_path);
 }
 
 static void
-contact_groups_file_parse (const gchar *filename) 
+contact_groups_file_parse (const gchar *filename)
 {
-	xmlParserCtxtPtr  ctxt;
-	xmlDocPtr         doc;
-	xmlNodePtr        contacts;
-	xmlNodePtr        account;
-	xmlNodePtr        node;
-	
-	DEBUG_MSG (("ContactGroups: Attempting to parse file:'%s'...", filename));
-	
- 	ctxt = xmlNewParserCtxt ();
+	xmlParserCtxtPtr ctxt;
+	xmlDocPtr        doc;
+	xmlNodePtr       contacts;
+	xmlNodePtr       account;
+	xmlNodePtr       node;
+
+	gossip_debug (DEBUG_DOMAIN, "Attempting to parse file:'%s'...", filename);
+
+	ctxt = xmlNewParserCtxt ();
 
 	/* Parse and validate the file. */
-	doc = xmlCtxtReadFile (ctxt, filename, NULL, 0);	
+	doc = xmlCtxtReadFile (ctxt, filename, NULL, 0);
 	if (!doc) {
 		g_warning ("Failed to parse file:'%s'", filename);
 		xmlFreeParserCtxt (ctxt);
@@ -138,13 +134,13 @@ contact_groups_file_parse (const gchar *filename)
 
 			name = (gchar *) xmlGetProp (node, "name");
 			expanded_str = (gchar *) xmlGetProp (node, "expanded");
-			
+
 			if (expanded_str && strcmp (expanded_str, "yes") == 0) {
 				expanded = TRUE;
 			} else {
 				expanded = FALSE;
 			}
-			
+
 			contact_group = contact_group_new (name, expanded);
 			groups = g_list_append (groups, contact_group);
 
@@ -154,9 +150,9 @@ contact_groups_file_parse (const gchar *filename)
 
 		node = node->next;
 	}
-	
-	DEBUG_MSG (("ContactGroups: Parsed %d contact groups", g_list_length (groups)));
-	  
+
+	gossip_debug (DEBUG_DOMAIN, "Parsed %d contact groups", g_list_length (groups));
+
 	xmlFreeDoc(doc);
 	xmlFreeParserCtxt (ctxt);
 }
@@ -168,7 +164,7 @@ contact_group_new (const gchar *name,
 	ContactGroup *group;
 
 	group = g_new0 (ContactGroup, 1);
-	
+
 	group->name = g_strdup (name);
 	group->expanded = expanded;
 
@@ -179,40 +175,31 @@ static void
 contact_group_free (ContactGroup *group)
 {
 	g_return_if_fail (group != NULL);
-	
+
 	g_free (group->name);
-	
+
 	g_free (group);
 }
 
 static gboolean
 contact_groups_file_save (void)
 {
-	xmlDocPtr   doc;  
-	xmlDtdPtr   dtd;  
+	xmlDocPtr   doc;
 	xmlNodePtr  root;
 	xmlNodePtr  node;
 	GList      *l;
-	gchar      *dtd_file;
 	gchar      *dir;
 	gchar      *file;
 
 	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
-	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
-		g_mkdir_with_parents (dir, S_IRUSR | S_IWUSR | S_IXUSR);
-	}
-
+	g_mkdir_with_parents (dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	file = g_build_filename (dir, CONTACT_GROUPS_XML_FILENAME, NULL);
 	g_free (dir);
 
-	dtd_file = g_build_filename (DTDDIR, CONTACT_GROUPS_DTD_FILENAME, NULL);
-
 	doc = xmlNewDoc ("1.0");
 	root = xmlNewNode (NULL, "contacts");
 	xmlDocSetRootElement (doc, root);
 
-	dtd = xmlCreateIntSubset (doc, "contacts", NULL, dtd_file);
-
 	node = xmlNewChild (root, NULL, "account", NULL);
 	xmlNewProp (node, "name", "Default");
 
@@ -221,22 +208,23 @@ contact_groups_file_save (void)
 		xmlNodePtr    subnode;
 
 		cg = l->data;
-		
+
 		subnode = xmlNewChild (node, NULL, "group", NULL);
-		xmlNewProp (subnode, "expanded", cg->expanded ? "yes" : "no");	
+		xmlNewProp (subnode, "expanded", cg->expanded ? "yes" : "no");
 		xmlNewProp (subnode, "name", cg->name);
 	}
 
-	DEBUG_MSG (("ContactGroups: Saving file:'%s'", file));
+	/* Make sure the XML is indented properly */
+	xmlIndentTreeOutput = 1;
+
+	gossip_debug (DEBUG_DOMAIN, "Saving file:'%s'", file);
 	xmlSaveFormatFileEnc (file, doc, "utf-8", 1);
 	xmlFreeDoc (doc);
 
 	xmlCleanupParser ();
-
 	xmlMemoryDump ();
-	
+
 	g_free (file);
-	g_free (dtd_file);
 
 	return TRUE;
 }
@@ -248,7 +236,7 @@ gossip_contact_group_get_expanded (const gchar *group)
 	gboolean  default_val = TRUE;
 
 	g_return_val_if_fail (group != NULL, default_val);
-	
+
 	for (l = groups; l; l = l->next) {
 		ContactGroup *cg = l->data;
 
@@ -264,8 +252,8 @@ gossip_contact_group_get_expanded (const gchar *group)
 	return default_val;
 }
 
-void    
-gossip_contact_group_set_expanded (const gchar *group, 
+void
+gossip_contact_group_set_expanded (const gchar *group,
 				   gboolean     expanded)
 {
 	GList        *l;
@@ -273,7 +261,7 @@ gossip_contact_group_set_expanded (const gchar *group,
 	gboolean      changed = FALSE;
 
 	g_return_if_fail (group != NULL);
-	
+
 	for (l = groups; l; l = l->next) {
 		ContactGroup *cg = l->data;
 
