@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2006 Imendio AB
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,21 +16,27 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Martyn Russell <martyn@imendio.com>
+ *          Richard Hult <richard@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 #include <stdio.h>
-#include <glade/glade.h>
+
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <glade/glade.h>
 
-#include <libgossip/gossip-chatroom-provider.h>
 #include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-chatroom-manager.h>
 
 #include "gossip-app.h"
 #include "gossip-edit-chatroom-dialog.h"
-
+#include "gossip-ui-utils.h"
 
 typedef struct {
 	GtkWidget      *dialog;
@@ -44,7 +50,6 @@ typedef struct {
 	GossipChatroom *chatroom;
 } GossipEditChatroomDialog;
 
-
 static void edit_chatroom_dialog_set              (GossipEditChatroomDialog *dialog);
 static void edit_chatroom_dialog_entry_changed_cb (GtkEntry                 *entry,
 						   GossipEditChatroomDialog *dialog);
@@ -54,7 +59,6 @@ static void edit_chatroom_dialog_response_cb      (GtkWidget                *wid
 static void edit_chatroom_dialog_destroy_cb       (GtkWidget                *widget,
 						   GossipEditChatroomDialog *dialog);
 
-
 static void
 edit_chatroom_dialog_set (GossipEditChatroomDialog *dialog)
 {
@@ -128,7 +132,9 @@ gossip_edit_chatroom_dialog_show (GtkWindow      *parent,
 				  GossipChatroom *chatroom)
 {
 	GossipEditChatroomDialog *dialog;
+	GossipAccount            *account;
 	GladeXML                 *glade;
+	GtkWidget                *label_server;
 
 	g_return_if_fail (chatroom != NULL);
 
@@ -142,6 +148,7 @@ gossip_edit_chatroom_dialog_show (GtkWindow      *parent,
 				       "edit_chatroom_dialog", &dialog->dialog,
 				       "entry_name", &dialog->entry_name,
 				       "entry_nickname", &dialog->entry_nickname,
+				       "label_server", &label_server,
 				       "entry_server", &dialog->entry_server,
 				       "entry_room", &dialog->entry_room,
 				       "checkbutton_auto_connect", &dialog->checkbutton_auto_connect,
@@ -160,6 +167,28 @@ gossip_edit_chatroom_dialog_show (GtkWindow      *parent,
 
 	g_object_unref (glade);
 
+	account = gossip_chatroom_get_account (chatroom);
+	switch (gossip_account_get_type (account)) {
+	case GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY:
+	case GOSSIP_ACCOUNT_TYPE_JABBER:
+		gtk_widget_show (label_server);
+		gtk_widget_show (dialog->entry_server);
+		break;
+
+	case GOSSIP_ACCOUNT_TYPE_AIM:
+	case GOSSIP_ACCOUNT_TYPE_ICQ:
+	case GOSSIP_ACCOUNT_TYPE_MSN:
+	case GOSSIP_ACCOUNT_TYPE_YAHOO:
+	case GOSSIP_ACCOUNT_TYPE_IRC:
+	case GOSSIP_ACCOUNT_TYPE_SALUT:
+		gtk_widget_hide (label_server);
+		gtk_widget_hide (dialog->entry_server);
+		break;
+
+	default:
+		break;
+	}
+
 	gtk_entry_set_text (GTK_ENTRY (dialog->entry_name),
 			    gossip_chatroom_get_name (chatroom));
 	gtk_entry_set_text (GTK_ENTRY (dialog->entry_nickname),
