@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2006 Imendio AB
+ * Copyright (C) 2006-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,21 +16,35 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
+
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <glade/glade.h>
 #include <libnotify/notify.h>
 
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-event.h>
+#include <libgossip/gossip-protocol.h>
+#include <libgossip/gossip-message.h>
+#include <libgossip/gossip-presence.h>
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-conf.h>
 
+#include "gossip-chat-manager.h"
 #include "gossip-app.h"
 #include "gossip-contact-info-dialog.h"
 #include "gossip-notify.h"
 #include "gossip-preferences.h"
 #include "gossip-stock.h"
+#include "gossip-ui-utils.h"
 
 #define DEBUG_DOMAIN "Notify"
 
@@ -81,7 +95,6 @@ static void                notify_contact_added_cb                (GossipSession
 static void                notify_contact_removed_cb              (GossipSession      *session,
 								   GossipContact      *contact,
 								   gpointer            user_data);
-static void                notify_contact_remove                  (GossipContact      *contact);
 static gboolean            notify_subscription_request_show_cb    (GossipContact      *contact);
 static void                notify_event_added_cb                  (GossipEventManager *event_manager,
 								   GossipEvent        *event,
@@ -143,7 +156,12 @@ static void
 notify_closed_cb (NotifyNotification *notify,
 		  gpointer            user_data)
 {
-	g_object_unref (notify);
+	/* NOTE: We only do this because libnotify<=0.4.3 breaks here
+	 * otherwise, this is all detailed in bug #395588.
+	 */
+#ifndef LIBNOTIFY_BUGGY_CLOSE
+	g_object_unref(notify);
+#endif
 }
 
 static gboolean
@@ -324,6 +342,31 @@ notify_new_message_default_cb (NotifyNotification *notify,
 	GossipMessage *message;
 	GossipContact *contact = NULL;
 
+	event = g_hash_table_lookup (event_notifications, notify);
+	if (event) {
+		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
+		contact = gossip_message_get_sender (message);
+		
+		gossip_chat_manager_remove_events(gossip_app_get_chat_manager(),contact);
+		
+		g_hash_table_remove (event_notifications, notify);
+		g_hash_table_remove (message_notifications, contact);
+	} else {
+		g_warning ("No event found for NotifyNotification: %p", notify);
+	}
+
+	g_object_unref (event_manager);
+}
+
+static void
+notify_new_message_respond_cb (NotifyNotification *notify,
+			       gchar              *label,
+			       GossipEventManager *event_manager)
+{
+	GossipEvent   *event;
+	GossipMessage *message;
+	GossipContact *contact = NULL;
+
 	event = g_hash_table_lookup (event_notifications, notify);
 	if (event) {
 		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
@@ -449,7 +492,7 @@ notify_new_message (GossipEventManager *event_manager,
 					(NotifyActionCallback) notify_new_message_default_cb,
 					g_object_ref (event_manager), NULL);
 	notify_notification_add_action (notify, "respond", _("Show"),
-					(NotifyActionCallback) notify_new_message_default_cb,
+					(NotifyActionCallback) notify_new_message_respond_cb,
 					g_object_ref (event_manager), NULL);
 
 	if (gossip_contact_get_type (contact) == GOSSIP_CONTACT_TYPE_TEMPORARY) {
@@ -541,15 +584,15 @@ notify_protocol_connected_cb (GossipSession  *session,
 			      gpointer        user_data)
 {
 	guint        id;
-	const gchar *account_id;
+	const gchar *account_name;
 
 	if (g_hash_table_lookup (account_states, account)) {
 		return;
 	}
 
-	account_id = gossip_account_get_id (account);
+	account_name = gossip_account_get_name (account);
 	gossip_debug (DEBUG_DOMAIN, "Account update, account:'%s' is now online",
-		      account_id);
+		      account_name);
 
 	id = g_timeout_add (NOTIFY_WAIT_TIME,
 			    (GSourceFunc) notify_protocol_timeout_cb,
@@ -616,17 +659,12 @@ static void
 notify_contact_removed_cb (GossipSession *session,
 			   GossipContact *contact,
 			   gpointer       user_data)
-{
-	g_hash_table_remove (contact_states, contact);
-}
-
-static void
-notify_contact_remove (GossipContact *contact)
 {
 	g_signal_handlers_disconnect_by_func (contact,
 					      notify_contact_presence_updated_cb,
 					      NULL);
-	g_object_unref (contact);
+
+	g_hash_table_remove (contact_states, contact);
 }
 
 static gboolean
@@ -844,7 +882,7 @@ gossip_notify_init (GossipSession      *session,
 
 	contact_states = g_hash_table_new_full (gossip_contact_hash,
 						gossip_contact_equal,
-						(GDestroyNotify) notify_contact_remove,
+						(GDestroyNotify) g_object_unref,
 						(GDestroyNotify) g_object_unref);
 
 	g_signal_connect (session, "protocol-connected",
