@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2006 Imendio AB
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,15 +16,20 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
+ * 
+ *          Part of this file is copied from GtkSourceView (gtksourceiter.c):
+ *          Paolo Maggi
+ *          Jeroen Zwartepoorte
  */
 
-/* Part of this file is copied from GtkSourceView (gtksourceiter.c):
- *  Copyright (C) 2000 - 2005 Paolo Maggi
- *  Copyright (C) 2002, 2003 Jeroen Zwartepoorte
- */
+#include "config.h"
 
-#include <config.h>
 #include <string.h>
+
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
@@ -37,15 +42,17 @@
 #include <Cocoa/Cocoa.h>
 #endif
 
+#include <libgossip/gossip-session.h>
+#include <libgossip/gossip-account-manager.h>
+#include <libgossip/gossip-conf.h>
+#include <libgossip/gossip-paths.h>
+
 #ifdef HAVE_LIBNOTIFY
 #include "gossip-notify.h"
 #endif
 
-#include <libgossip/gossip-conf.h>
-#include <libgossip/gossip-paths.h>
-
-#include "gossip-stock.h"
 #include "gossip-app.h"
+#include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 
 struct SizeData {
@@ -228,7 +235,7 @@ gossip_password_dialog_run (GossipAccount *account,
 	GtkWidget   *vbox;
 	gchar       *str;
 	gchar       *password;
-	const gchar *id;
+	const gchar *name;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
 
@@ -242,8 +249,8 @@ gossip_password_dialog_run (GossipAccount *account,
 						     str);
 	g_free (str);
 
-	id = gossip_account_get_id (account);
-	str = g_strdup_printf (_("Logging in with: %s"), id);
+	name = gossip_account_get_name (account);
+	str = g_strdup_printf (_("Logging in to account '%s'"), name);
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", str);
 	g_free (str);
 
@@ -278,7 +285,7 @@ gossip_password_dialog_run (GossipAccount *account,
 			session = gossip_app_get_session ();
 			manager = gossip_session_get_account_manager (session);
 
-			gossip_account_param_set (account, "password", password, NULL);
+			gossip_account_set_param (account, "password", password, NULL);
 			gossip_account_manager_store (manager);
 		}
 	} else {
@@ -417,6 +424,7 @@ gossip_pixbuf_from_account_type (GossipAccountType type,
 	}
 
 	switch (type) {
+	case GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY:
 	case GOSSIP_ACCOUNT_TYPE_JABBER:
 		icon_id = "im-jabber";
 		break;
@@ -432,18 +440,20 @@ gossip_pixbuf_from_account_type (GossipAccountType type,
 	case GOSSIP_ACCOUNT_TYPE_YAHOO:
 		icon_id = "im-yahoo";
 		break;
+
+	/* FIXME: we should have an artwork for these protocols */
+	case GOSSIP_ACCOUNT_TYPE_IRC:
+	case GOSSIP_ACCOUNT_TYPE_SALUT:
 	case GOSSIP_ACCOUNT_TYPE_UNKNOWN:
-		/* FIXME: we should have an artwork for unknown protocols */
-		icon_id = "im-jabber";
-		break;
 	default:
-		g_assert_not_reached ();
+		icon_id = "im";
+		break;
 	}
 
 	pixbuf = gtk_icon_theme_load_icon (theme,
-					   icon_id,     /* icon name */
-					   size,        /* size */
-					   0,           /* flags */
+					   icon_id,     /* Icon name */
+					   size,        /* Size */
+					   0,           /* Flags */
 					   &error);
 
 	return pixbuf;
