@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004-2006 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,42 +16,50 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 #include <unistd.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <glib/gi18n.h>
 
 #ifdef HAVE_GNOME
 #include <libgnomeui/libgnomeui.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #endif
 
-#include <libgossip/gossip-debug.h>
-#include <libgossip/gossip-conf.h>
-#include <libgossip/gossip-protocol.h>
 #include <libgossip/gossip-session.h>
+#include <libgossip/gossip-async.h>
+#include <libgossip/gossip-protocol.h>
+#include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-avatar.h>
 #include <libgossip/gossip-utils.h>
 #include <libgossip/gossip-vcard.h>
+#include <libgossip/gossip-conf.h>
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-debug.h>
 
 #include "gossip-account-chooser.h"
 #include "gossip-app.h"
 #include "gossip-vcard-dialog.h"
 #include "gossip-image-chooser.h"
 #include "gossip-preferences.h"
+#include "gossip-popup-button.h"
+#include "gossip-ui-utils.h"
 
 #define DEBUG_DOMAIN "VCardDialog"
 
-#define STRING_EMPTY(x) ((x) == NULL || (x)[0] == '\0')
-
 #define VCARD_TIMEOUT     20000
 #define SAVED_TIMEOUT     10000
 
-#define RESPONSE_SAVE     1
-
 #define AVATAR_MAX_HEIGHT 96
 #define AVATAR_MAX_WIDTH  96
 
@@ -62,7 +70,7 @@ typedef struct {
 	GtkWidget *label_account;
 	GtkWidget *account_chooser;
 
-	GtkWidget *table_vcard;
+	GtkWidget *vbox_details;
 
 	GtkWidget *label_name;
 	GtkWidget *label_nickname;
@@ -77,6 +85,11 @@ typedef struct {
 
 	GtkWidget *textview_description;
 
+	GtkWidget *entry_birthday;
+	GtkWidget *button_birthday;
+
+	GtkWidget *calendar;
+
 	GtkWidget *button_cancel;
 	GtkWidget *button_save;
 
@@ -86,13 +99,14 @@ typedef struct {
 	guint      timeout_id;
 	gboolean   requesting_vcard;
 	gint       last_account_selected;
+	gchar     *avatar_format;
 
 #ifdef HAVE_GNOME
 	GnomeThumbnailFactory *thumbs;
 #endif
 } GossipVCardDialog;
 
-static GtkWidget *vcard_dialog_create_avatar_chooser      (void);
+static void       vcard_dialog_create_avatar_chooser      (GossipVCardDialog *dialog);
 static void       vcard_dialog_avatar_chooser_response_cb (GtkWidget         *widget,
 							   gint               response,
 							   GossipVCardDialog *dialog);
@@ -100,6 +114,18 @@ static void       vcard_dialog_avatar_clicked_cb          (GtkWidget         *bu
 							   GossipVCardDialog *dialog);
 static void       vcard_dialog_avatar_update_preview_cb   (GtkFileChooser    *chooser,
 							   GossipVCardDialog *dialog);
+static GtkWidget *vcard_dialog_birthday_button_popup_cb   (GossipPopupButton *popup_button,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_birthday_button_popdown_cb (GossipPopupButton *popup_button,
+							   GtkWidget         *widget,
+							   gboolean           ok,
+							   GossipVCardDialog *dialog);
+static gboolean   vcard_dialog_birthday_parse_string      (const gchar       *str,
+							   gint              *year,
+							   gint              *month,
+							   gint              *day);
+static gchar *    vcard_dialog_birthday_string_to_server  (const gchar       *str);
+static gchar *    vcard_dialog_birthday_string_from_server(const gchar       *str);
 static void       vcard_dialog_set_account_to_last        (GossipVCardDialog *dialog);
 static void       vcard_dialog_lookup_start               (GossipVCardDialog *dialog);
 static void       vcard_dialog_lookup_stop                (GossipVCardDialog *dialog);
@@ -118,27 +144,41 @@ static void       vcard_dialog_response_cb                (GtkDialog         *wi
 static void       vcard_dialog_destroy_cb                 (GtkWidget         *widget,
 							   GossipVCardDialog *dialog);
 
-
 static GossipVCardDialog *dialog = NULL;
 
-static GtkWidget *
-vcard_dialog_create_avatar_chooser (void)
+static void
+vcard_dialog_create_avatar_chooser (GossipVCardDialog *dialog)
 {
-	GtkWidget *chooser;
+	GossipAccountChooser *account_chooser;
+	GossipAccount        *account;
+	guint                 min_width;
+	guint                 min_height;
+	guint                 max_width;
+	guint                 max_height;
+	gsize                 max_size;
 
-	chooser = gossip_image_chooser_new ();
+	dialog->avatar_chooser = gossip_image_chooser_new ();
+	account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
+	account = gossip_account_chooser_get_account (account_chooser);
 
-	gossip_image_chooser_set_image_max_size (GOSSIP_IMAGE_CHOOSER (chooser),
-						 AVATAR_MAX_WIDTH,
-						 AVATAR_MAX_HEIGHT);
+	gossip_session_get_avatar_requirements (gossip_app_get_session (),
+						account,
+						&min_width, &min_height,
+						&max_width, &max_height,
+						&max_size,  &dialog->avatar_format);
 
-	gtk_widget_set_size_request (chooser,
+	gossip_image_chooser_set_requirements (GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser),
+					       min_width, min_height,
+					       max_width, max_height,
+					       max_size,  dialog->avatar_format);
+
+	gtk_widget_set_size_request (dialog->avatar_chooser,
 				     AVATAR_MAX_WIDTH / 2,
 				     AVATAR_MAX_HEIGHT / 2);
 
-	gtk_widget_show_all (chooser);
-
-	return chooser;
+	gtk_container_add (GTK_CONTAINER (dialog->button_image),
+			   dialog->avatar_chooser);
+	gtk_widget_show_all (dialog->avatar_chooser);
 }
 
 static void
@@ -162,7 +202,8 @@ vcard_dialog_avatar_chooser_response_cb (GtkWidget         *widget,
 						 path);
 			g_free (path);
 		}
-	} else if (response == GTK_RESPONSE_NO) {
+	}
+	else if (response == GTK_RESPONSE_NO) {
 		gossip_image_chooser_set_image_data
 			(GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser), NULL, 0);
 	}
@@ -294,6 +335,171 @@ vcard_dialog_avatar_update_preview_cb (GtkFileChooser    *chooser,
 #endif
 }
 
+
+static void
+vcard_dialog_birthday_cancel_clicked_cb (GtkWidget         *button,
+					 GossipPopupButton *popup_button)
+{
+	gossip_popup_button_popdown (popup_button, FALSE);
+}
+
+static void
+vcard_dialog_birthday_ok_clicked_cb (GtkWidget         *button,
+				     GossipPopupButton *popup_button)
+{
+	gossip_popup_button_popdown (popup_button, TRUE);
+}
+
+static gboolean
+vcard_dialog_birthday_parse_string (const gchar *str,
+				    gint        *year,
+				    gint        *month,
+				    gint        *day)
+{
+	GDate *date;
+
+	date = g_date_new ();
+	g_date_set_parse (date, str);
+
+	if (!g_date_valid (date)) {
+		g_date_free (date);
+		return FALSE;
+	}
+
+	*year = g_date_get_year (date);
+	*month = g_date_get_month (date);
+	*day = g_date_get_day (date);
+
+	g_date_free (date);
+
+	return TRUE;
+}
+
+static gchar *
+vcard_dialog_birthday_string_to_server (const gchar *str)
+{
+	gint year, month, day;
+
+	if (vcard_dialog_birthday_parse_string (str, &year, &month, &day)) {
+		return g_strdup_printf ("%04d-%02d-%02d", year, month, day);
+	}
+
+	return NULL;
+}
+
+static gchar *
+vcard_dialog_birthday_string_from_server (const gchar *str)
+{
+	gint         year, month, day;
+	GDate       *date;
+	const gchar *format = "%x"; /* Keep in variable get rid of warning. */
+	gchar        buf[128];
+
+	if (sscanf (str, "%04d-%02d-%02d", &year, &month, &day) != 3) {
+		return NULL;
+	}
+
+	date = g_date_new ();
+	g_date_set_dmy (date, day, month, year);
+
+	if (g_date_strftime (buf, sizeof (buf), format, date) > 0) {
+		g_date_free (date);
+		return g_strdup (buf);
+	}
+
+	g_date_free (date);
+
+	return NULL;
+}
+
+static void
+vcard_dialog_birthday_double_click_cb (GtkWidget         *calendar,
+				       GossipPopupButton *popup_button)
+{
+	gossip_popup_button_popdown (popup_button, TRUE);
+}
+
+static void
+vcard_dialog_birthday_button_popdown_cb (GossipPopupButton *button,
+					 GtkWidget         *widget,
+					 gboolean           ok,
+					 GossipVCardDialog *dialog)
+{
+	gint         year, month, day;
+	GDate       *date;
+	const gchar *format = "%x"; /* Keep in variable get rid of warning. */
+	gchar        buf[128];
+
+	if (ok) {
+		gtk_calendar_get_date (GTK_CALENDAR (dialog->calendar),
+				       &year, &month, &day);
+
+		date = g_date_new ();
+		g_date_set_dmy (date, day, month + 1, year);
+
+		if (g_date_strftime (buf, sizeof (buf), format, date) > 0) {
+			gtk_entry_set_text (GTK_ENTRY (dialog->entry_birthday), buf);
+		}
+
+		g_date_free (date);
+	}
+
+	dialog->calendar = NULL;
+	gtk_widget_destroy (widget);
+}
+
+static GtkWidget *
+vcard_dialog_birthday_button_popup_cb (GossipPopupButton *popup_button,
+				       GossipVCardDialog *dialog)
+{
+	GtkWidget   *frame;
+	GtkWidget   *vbox;
+	GtkWidget   *bbox;
+	GtkWidget   *button;
+	const gchar *str;
+	gint         year, month, day;
+
+	frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
+
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_container_add (GTK_CONTAINER (frame), vbox);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
+
+	dialog->calendar = gtk_calendar_new ();
+	gtk_box_pack_start (GTK_BOX (vbox), dialog->calendar, TRUE, TRUE, 0);
+
+	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_birthday));
+	if (vcard_dialog_birthday_parse_string (str, &year, &month, &day)) {
+		gtk_calendar_select_month (GTK_CALENDAR (dialog->calendar), month - 1, year);
+		gtk_calendar_select_day (GTK_CALENDAR (dialog->calendar), day);
+	}
+
+	bbox = gtk_hbutton_box_new ();
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
+	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
+
+	button = gtk_button_new_with_label (_("Cancel"));
+	gtk_container_add (GTK_CONTAINER (bbox), button);
+	g_signal_connect (button,
+			  "clicked",
+			  G_CALLBACK (vcard_dialog_birthday_cancel_clicked_cb),
+			  popup_button);
+
+	button = gtk_button_new_with_label (_("OK"));
+	gtk_container_add (GTK_CONTAINER (bbox), button);
+	g_signal_connect (button, "clicked",
+			  G_CALLBACK (vcard_dialog_birthday_ok_clicked_cb),
+			  popup_button);
+
+	g_signal_connect (dialog->calendar,
+			  "day-selected-double-click",
+			  G_CALLBACK (vcard_dialog_birthday_double_click_cb),
+			  popup_button);
+
+	return frame;
+}
+
 static void
 vcard_dialog_set_account_to_last (GossipVCardDialog *dialog)
 {
@@ -316,7 +522,7 @@ vcard_dialog_lookup_start (GossipVCardDialog *dialog)
 	GossipAccount        *account;
 	GossipAccountChooser *account_chooser;
 
-	gtk_widget_set_sensitive (dialog->table_vcard, FALSE);
+	gtk_widget_set_sensitive (dialog->vbox_details, FALSE);
 	gtk_widget_set_sensitive (dialog->account_chooser, FALSE);
 	gtk_widget_set_sensitive (dialog->button_save, FALSE);
 
@@ -328,6 +534,8 @@ vcard_dialog_lookup_start (GossipVCardDialog *dialog)
 	account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
 	account = gossip_account_chooser_get_account (account_chooser);
 
+	dialog->requesting_vcard = TRUE;
+
 	gossip_session_get_vcard (session,
 				  account,
 				  NULL,
@@ -335,8 +543,6 @@ vcard_dialog_lookup_start (GossipVCardDialog *dialog)
 				  dialog,
 				  NULL);
 
-	dialog->requesting_vcard = TRUE;
-
 	g_object_unref (account);
 }
 
@@ -345,7 +551,7 @@ vcard_dialog_lookup_stop (GossipVCardDialog *dialog)
 {
 	dialog->requesting_vcard = FALSE;
 
-	gtk_widget_set_sensitive (dialog->table_vcard, TRUE);
+	gtk_widget_set_sensitive (dialog->vbox_details, TRUE);
 	gtk_widget_set_sensitive (dialog->account_chooser, TRUE);
 	gtk_widget_set_sensitive (dialog->button_save, TRUE);
 
@@ -363,8 +569,7 @@ vcard_dialog_get_vcard_cb (GossipResult       result,
 	GtkComboBox   *combo_box;
 	GtkTextBuffer *buffer;
 	const gchar   *str;
-	const guchar  *avatar;
-	gsize          avatar_size;
+	GossipAvatar  *avatar;
 
 	gossip_debug (DEBUG_DOMAIN, "Received VCard response");
 
@@ -380,25 +585,39 @@ vcard_dialog_get_vcard_cb (GossipResult       result,
 	}
 
 	str = gossip_vcard_get_name (vcard);
-	gtk_entry_set_text (GTK_ENTRY (dialog->entry_name), STRING_EMPTY (str) ? "" : str);
+	gtk_entry_set_text (GTK_ENTRY (dialog->entry_name), G_STR_EMPTY (str) ? "" : str);
 
 	str = gossip_vcard_get_nickname (vcard);
-	gtk_entry_set_text (GTK_ENTRY (dialog->entry_nickname), STRING_EMPTY (str) ? "" : str);
+	gtk_entry_set_text (GTK_ENTRY (dialog->entry_nickname), G_STR_EMPTY (str) ? "" : str);
+
+	str = gossip_vcard_get_birthday (vcard);
+	if (str) {
+		gchar *date;
+
+		date = vcard_dialog_birthday_string_from_server (str);
+		gtk_entry_set_text (GTK_ENTRY (dialog->entry_birthday), G_STR_EMPTY (date) ? "" : date);
+		g_free (date);
+	}
 
 	str = gossip_vcard_get_email (vcard);
-	gtk_entry_set_text (GTK_ENTRY (dialog->entry_email), STRING_EMPTY (str) ? "" : str);
+	gtk_entry_set_text (GTK_ENTRY (dialog->entry_email), G_STR_EMPTY (str) ? "" : str);
 
 	str = gossip_vcard_get_url (vcard);
-	gtk_entry_set_text (GTK_ENTRY (dialog->entry_web_site), STRING_EMPTY (str) ? "" : str);
+	gtk_entry_set_text (GTK_ENTRY (dialog->entry_web_site), G_STR_EMPTY (str) ? "" : str);
 
 	str = gossip_vcard_get_description (vcard);
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (dialog->textview_description));
-	gtk_text_buffer_set_text (buffer, STRING_EMPTY (str) ? "" : str, -1);
+	gtk_text_buffer_set_text (buffer, G_STR_EMPTY (str) ? "" : str, -1);
 
-	avatar = gossip_vcard_get_avatar (vcard, &avatar_size);
+	avatar = gossip_vcard_get_avatar (vcard);
 
-	gossip_image_chooser_set_image_data (GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser),
-					     (gchar*) avatar, avatar_size);
+	if (avatar) {
+		gossip_image_chooser_set_image_data (GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser),
+						     (gchar*) avatar->data, avatar->len);
+	} else {
+		gossip_image_chooser_set_image_data (GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser),
+						     NULL, 0);
+	}
 
 	/* Save position incase the next lookup fails. */
 	combo_box = GTK_COMBO_BOX (dialog->account_chooser);
@@ -431,8 +650,10 @@ vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 	GtkTextIter           iter_begin, iter_end;
 	gchar                *description;
 	const gchar          *str;
-	gchar                *avatar;
+	gchar                *avatar_data;
 	gsize                 avatar_size;
+	GossipAvatar         *avatar = NULL;
+	gchar                *birthday;
 
 	if (!gossip_app_is_connected ()) {
 		gossip_debug (DEBUG_DOMAIN, "Not connected, not setting VCard");
@@ -447,6 +668,13 @@ vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_nickname));
 	gossip_vcard_set_nickname (vcard, str);
 
+	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_birthday));
+	birthday = vcard_dialog_birthday_string_to_server (str);
+	if (birthday) {
+		gossip_vcard_set_birthday (vcard, birthday);
+		g_free (birthday);
+	}
+
 	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_web_site));
 	gossip_vcard_set_url (vcard, str);
 
@@ -460,8 +688,11 @@ vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 	g_free (description);
 
 	gossip_image_chooser_get_image_data (GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser),
-					     &avatar, &avatar_size);
-	gossip_vcard_set_avatar (vcard, avatar, avatar_size);
+					     &avatar_data, &avatar_size);
+	if (avatar_data) {
+		avatar = gossip_avatar_new (avatar_data, avatar_size, dialog->avatar_format);
+		gossip_vcard_set_avatar (vcard, avatar);
+	}
 
 	/* NOTE: if account is NULL, all accounts will get the same vcard */
 	account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
@@ -470,14 +701,16 @@ vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 	gossip_session_set_vcard (gossip_app_get_session (),
 				  account,
 				  vcard,
-				  (GossipResultCallback) vcard_dialog_set_vcard_cb,
+				  (GossipCallback) vcard_dialog_set_vcard_cb,
 				  NULL, &error);
 
 	protocol = gossip_session_get_protocol (gossip_app_get_session (), account);
 	contact = gossip_protocol_get_own_contact (protocol);
-	gossip_contact_set_avatar (GOSSIP_CONTACT(contact), avatar, avatar_size);
+	gossip_contact_set_avatar (GOSSIP_CONTACT (contact), avatar);
 
-	g_free (avatar);
+	if (avatar) {
+		gossip_avatar_unref (avatar);
+	}
 
 	g_object_unref (account);
 }
@@ -528,7 +761,7 @@ vcard_dialog_response_cb (GtkDialog         *widget,
 			  GossipVCardDialog *dialog)
 {
 	switch (response) {
-	case RESPONSE_SAVE:
+	case GTK_RESPONSE_OK:
 		gtk_widget_set_sensitive (dialog->button_save, FALSE);
 		vcard_dialog_set_vcard (dialog);
 		return;
@@ -566,7 +799,7 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 	GossipSession *session;
 	GladeXML      *glade;
 	GList         *accounts;
-	GtkSizeGroup  *size_group;
+	GtkWidget     *birthday_placeholder;
 
 	if (dialog) {
 		gtk_window_present (GTK_WINDOW (dialog->dialog));
@@ -581,7 +814,7 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 				       "vcard_dialog", &dialog->dialog,
 				       "hbox_account", &dialog->hbox_account,
 				       "label_account", &dialog->label_account,
-				       "table_vcard", &dialog->table_vcard,
+				       "vbox_details", &dialog->vbox_details,
 				       "label_name", &dialog->label_name,
 				       "label_nickname", &dialog->label_nickname,
 				       "label_web_site", &dialog->label_web_site,
@@ -591,6 +824,8 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 				       "entry_nickname", &dialog->entry_nickname,
 				       "entry_web_site", &dialog->entry_web_site,
 				       "entry_email", &dialog->entry_email,
+				       "entry_birthday", &dialog->entry_birthday,
+				       "box_birthday_placeholder", &birthday_placeholder,
 				       "textview_description", &dialog->textview_description,
 				       "button_cancel", &dialog->button_cancel,
 				       "button_save", &dialog->button_save,
@@ -601,29 +836,38 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 			      dialog,
 			      "vcard_dialog", "destroy", vcard_dialog_destroy_cb,
 			      "vcard_dialog", "response", vcard_dialog_response_cb,
+			      "button_image", "clicked", vcard_dialog_avatar_clicked_cb,
 			      NULL);
 
 	g_object_add_weak_pointer (G_OBJECT (dialog->dialog), (gpointer) &dialog);
+
+	gossip_glade_setup_size_group (glade,
+				       GTK_SIZE_GROUP_HORIZONTAL,
+				       "label_account",
+				       "label_name",
+				       "label_nickname",
+				       "label_email",
+				       "label_web_site",
+				       "label_birthday",
+				       "label_avatar",
+				       "label_description",
+				       NULL);
+
 	g_object_unref (glade);
 
-	dialog->avatar_chooser = vcard_dialog_create_avatar_chooser ();
-	gtk_container_add (GTK_CONTAINER (dialog->button_image), dialog->avatar_chooser);
+	/* Birthday chooser */
+	dialog->button_birthday = gossip_popup_button_new (_("Change"));
+	gtk_box_pack_start (GTK_BOX (birthday_placeholder), dialog->button_birthday, 0, FALSE, FALSE);
+	gtk_widget_show (dialog->button_birthday);
 
-	/* Line up widgets */
-	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	g_signal_connect (dialog->button_birthday,
+			  "popup",
+			  G_CALLBACK (vcard_dialog_birthday_button_popup_cb),
+			  dialog);
 
-	gtk_size_group_add_widget (size_group, dialog->label_account);
-	gtk_size_group_add_widget (size_group, dialog->label_name);
-	gtk_size_group_add_widget (size_group, dialog->label_nickname);
-	gtk_size_group_add_widget (size_group, dialog->label_email);
-	gtk_size_group_add_widget (size_group, dialog->label_web_site);
-	gtk_size_group_add_widget (size_group, dialog->label_description);
-
-	g_object_unref (size_group);
-
-	/* Setup image chooser */
-	g_signal_connect (dialog->button_image, "clicked",
-			  G_CALLBACK (vcard_dialog_avatar_clicked_cb),
+	g_signal_connect (dialog->button_birthday,
+			  "popdown",
+			  G_CALLBACK (vcard_dialog_birthday_button_popdown_cb),
 			  dialog);
 
 	/* Sort out accounts */
@@ -640,6 +884,9 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 
 	gtk_widget_show (dialog->account_chooser);
 
+	/* Create the avatar chooser */
+	vcard_dialog_create_avatar_chooser (dialog);
+
 	/* Select first */
 	accounts = gossip_session_get_accounts (session);
 	if (g_list_length (accounts) > 1) {
@@ -661,5 +908,3 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 
 	vcard_dialog_lookup_start (dialog);
 }
-
-
